Package: GUniFrac
Type: Package
Title: Generalized UniFrac Distances, Distance-Based Multivariate
        Methods and Feature-Based Univariate Methods for Microbiome
        Data Analysis
Version: 1.9
Date: 2025-08-25
Authors@R: c(person(given = "Jun",
                        family = "Chen",
                        role = c("aut", "cre"),
                        email = "chen.jun2@mayo.edu"),
                 person(given = "Xianyang",
                        family = "Zhang",
                        role = "aut"),
                 person(given = "Lu",
                        family = "Yang",
                        role = "aut"),
                 person(given = "Lujun",
                        family = "Zhang",
                        role = "aut"))
Description: A suite of methods for powerful and robust microbiome data analysis including data normalization, data simulation, community-level association testing and differential abundance analysis. It implements generalized UniFrac distances,  Geometric Mean of Pairwise Ratios (GMPR) normalization, semiparametric data simulator, distance-based statistical methods, and feature-based statistical methods. The distance-based statistical methods include three extensions of PERMANOVA: (1) PERMANOVA using the Freedman-Lane permutation scheme, (2) PERMANOVA omnibus test using multiple matrices, and  (3) analytical approach to approximating PERMANOVA p-value. Feature-based statistical methods include linear model-based methods for differential abundance analysis of zero-inflated high-dimensional compositional data. 
Depends: R (>= 3.5.0)
Suggests: ade4, knitr, markdown, ggpubr
Imports: Rcpp (>= 0.12.13), vegan, ggplot2, matrixStats, Matrix, ape,
        parallel, stats, utils, statmod, rmutil, dirmult, MASS,
        ggrepel, foreach, modeest, inline, methods
LinkingTo: Rcpp
NeedsCompilation: yes
VignetteBuilder: knitr
License: GPL-3
Encoding: UTF-8
Packaged: 2025-08-25 19:13:10 UTC; chen.jun2
Author: Jun Chen [aut, cre],
  Xianyang Zhang [aut],
  Lu Yang [aut],
  Lujun Zhang [aut]
Maintainer: Jun Chen <chen.jun2@mayo.edu>
Repository: CRAN
Date/Publication: 2025-08-25 21:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 15:31:44 UTC; unix
Archs: GUniFrac.so.dSYM

importFrom(vegan, adonis, vegdist)
importFrom(ape, is.rooted, drop.tip)
import(statmod)
import(matrixStats)
import(stats)
importFrom(inline, cxxfunction)
importFrom(methods, signature)
importFrom(Rcpp, evalCpp)
importFrom(rmutil, dbetabinom)
importFrom(Matrix, nearPD)
importFrom(MASS, rnegbin)
importFrom(parallel, mclapply, makeCluster, stopCluster, clusterEvalQ,
           parApply, parLapply, parSapply, parRapply, parCapply)
importFrom(utils, getFromNamespace)   
import(foreach)
import(ggplot2)
import(ggrepel)
importFrom(modeest, mlv)
importFrom("grDevices", "dev.off", "hcl", "pdf")

export(GUniFrac)
export(Rarefy)
export(PermanovaG)
export(PermanovaG2)
export(SimulateMSeq)
export(SimulateMSeqC)
export(dmanova)
export(adonis3)
export(ZicoSeq)
export(GMPR)
export(ZicoSeq.plot)
export(dICC)
export(dICC.SE.asympt)
export(dICC.SE.bt)

useDynLib(GUniFrac)

## -----------------------------------------------------------------------------
# install.packages("GUniFrac")

## ----message=FALSE------------------------------------------------------------
library(GUniFrac)

## ----results = FALSE----------------------------------------------------------
data(throat.otu.tab)
data(throat.meta)
comm <- t(throat.otu.tab)
meta.dat <- throat.meta
meta.dat


## ----results = FALSE----------------------------------------------------------
ZicoSeq.obj <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm, 
                     grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "count",
                    # Filter to remove rare taxa
                    prev.filter = 0.2, mean.abund.filter = 0,  
                    max.abund.filter = 0.002, min.prop = 0, 
                    # Winsorization to replace outliers
                    is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
                    # Posterior sampling 
                    is.post.sample = TRUE, post.sample.no = 25, 
                    # Use the square-root transformation
                    link.func = list(function (x) x^0.5), stats.combine.func = max,
                    # Permutation-based multiple testing correction
                    perm.no = 99,  strata = NULL, 
                    # Reference-based multiple stage normalization
                    ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
                    # Family-wise error rate control
                    is.fwer = TRUE, verbose = TRUE, return.feature.dat = TRUE)

## ----fig.retina = 4, fig.width= 8, fig.height=6, results=FALSE, message=FALSE, warning=FALSE----

ZicoSeq.plot(ZicoSeq.obj, pvalue.type = 'p.adj.fdr', cutoff = 0.1, text.size = 10,
             out.dir = NULL, width = 10, height = 6)

## ----results = FALSE,  fig.width=8, fig.height=6------------------------------
comm.p <- t(t(comm) / colSums(comm))
ZicoSeq.obj.p <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm.p, 
                    grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "proportion",
                    # Filter to remove rare taxa
                    prev.filter = 0.2, mean.abund.filter = 0,  max.abund.filter = 0.002, min.prop = 0, 
                    # Winsorization to replace outliers
                    is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
                    # Posterior sampling will be automatically disabled
                    is.post.sample = FALSE, post.sample.no = 25, 
                    # Use the square-root transformation
                    link.func = list(function (x) x^0.5, function (x) x^0.25), stats.combine.func = max,
                    # Permutation-based multiple testing correction
                    perm.no = 99,  strata = NULL, 
                    # Reference-based multiple stage normalization
                    ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
                    # Family-wise error rate control
                    is.fwer = TRUE, verbose = TRUE, return.feature.dat = T)
suppressWarnings(ZicoSeq.plot(ZicoSeq.obj = ZicoSeq.obj.p, pvalue.type = 'p.adj.fdr', 
             cutoff = 0.1, text.size = 10, out.dir = NULL, width = 10, height = 6))                    


## ----fig.retina = 4, fig.width= 8, fig.height=6, results=FALSE, message=FALSE, warning=FALSE----
comm.o <- comm[rowMeans(comm != 0) >= 0.2, ] + 1
comm.o <- log(t(t(comm.o) / colSums(comm.o)))
ZicoSeq.obj.o <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm.o, 
		grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "other",
		# Filter will not be applied
		prev.filter = 0, mean.abund.filter = 0,  max.abund.filter = 0, min.prop = 0, 
		# Winsorization the top end 
		is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
		# Posterior sampling will be automatically disabled
		is.post.sample = FALSE, post.sample.no = 25, 
		# Identity function is used
		link.func = list(function (x) x), stats.combine.func = max,
		# Permutation-based multiple testing correction
		perm.no = 99,  strata = NULL, 
		# Reference-based multiple-stage normalization will not be performed
		ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
		# Family-wise error rate control
		is.fwer = TRUE, verbose = TRUE, return.feature.dat = T)
    ZicoSeq.plot(ZicoSeq.obj = ZicoSeq.obj.o,  pvalue.type = 'p.adj.fdr', 
             cutoff = 0.1, text.size = 10, out.dir = NULL, width = 10, height = 6)    

## -----------------------------------------------------------------------------
sessionInfo()

GMPR <- function(OTUmatrix, min_ct=2, intersect_no=4){
  #datatype check
	
  if(!(class(OTUmatrix)[1] %in% c("data.frame","matrix")))
    stop("Unknown datatype of object \"OTUmatrix\".")
   OTUmatrix  <- t(OTUmatrix)
  rownames(OTUmatrix)->SampleName
  if(ncol(OTUmatrix)<nrow(OTUmatrix))
    warning("Sample size is larger than OTU number. Check if samples are arranged in columns.")
  if(length(OTUmatrix[OTUmatrix<1 & OTUmatrix>0])>=0.1*ncol(OTUmatrix)*nrow(OTUmatrix))
    stop("More than 10% values are fractional, please check.")
  apply(OTUmatrix, MARGIN = 2, as.integer)->OTUmatrix


  gmpr(OTUmatrix,min_ct,intersect_no)->size.factor
  names(size.factor)<-SampleName
  size.factor[abs(size.factor-1)<1e-10]<-NA
  size.factor
}

#' @title estimiate the DM parameters
#' @param alpha shape factor
#' @return a matrix represents the true proportion of the ref.otu.tab, row is otu, column is sample
#' @import stats
#' @rdname rdirichlet.m
#' @export

rdirichlet.m <- function (alpha) {
	Gam <- matrix(rgamma(length(alpha), shape = alpha), nrow(alpha), ncol(alpha))
	t(t(Gam) / colSums(Gam))
}



#' @title estimiate the true proportion of ref.otu.tab
#' @param ref.otu.tab shape factor
#' @return A list with the elements
#' \item{mu}{absolute abundance before applying size factor}
#' \item{ref.otu.tab}{absolute abundance renormalized by size factor}
#' @rdname EstPara
#' @import dirmult
#' @import stats
#' @export

EstPara <- function (ref.otu.tab) {
	
	if (is.null(rownames(ref.otu.tab))) {
		rownames(ref.otu.tab) <- paste0('OTU', 1 : nrow(ref.otu.tab))
	} # otu * sample
	samplenames = colnames(ref.otu.tab)
	taxnames = rownames(ref.otu.tab)
	
	dirmult.paras <- dirmult::dirmult(t(ref.otu.tab))
	
	gamma = dirmult.paras$gamma
	names(gamma) = names(dirmult.paras$pi)
	
	# Add pseduo count(each OTU add gamma estimated from dirmult)
	ref.otu.tab = sapply(1:ncol(ref.otu.tab), function (i) gamma + ref.otu.tab[,i]) # C_ij otu * sample
	
	# back to dirchlet, calculate the true proportion
	ref.otu.tab.p <- rdirichlet.m(ref.otu.tab) # P_ij nOTU*nSam
	colnames(ref.otu.tab.p) = samplenames
	rownames(ref.otu.tab.p) = taxnames
	
	# order OTUs by mean OTU proportion, for later selection
	ord = order(rowMeans(ref.otu.tab.p), decreasing = TRUE)
	ref.otu.tab.p =  ref.otu.tab.p[ord,]
	
	# apply size factor
	Si = exp(rnorm(ncol(ref.otu.tab.p)))
	ref.otu.tab0 = t(t(ref.otu.tab.p)*Si)
	colnames(ref.otu.tab0) = colnames(ref.otu.tab.p)
	return(list(mu = ref.otu.tab.p, ref.otu.tab = ref.otu.tab0))
}





#' @title Simulate microbiome data for differential abundance analysis
#' @param ref.otu.tab taxa in rows, and samples in columns.
#' @param nSam an integer: indicating the number of samples you want to simulate.
#' @param nOTU an integer: indicating the number of taxa you want to simulate.
#' @param diff.otu.pct a number between 0-1: indicating the proportion of differential taxa, default is 0.
#' @param diff.otu.direct a character: consists "balanced" and "unbalanced". "balanced" is default, means differential taxa increase and decrease, "unbalanced" means differetial taxa only increase or only decrease.
#' @param diff.otu.mode a character: consists "rare", "mix", "abundant". "rare" means differential taxa come from the least abundant 0.25 taxa. "abundant", is default, means the most abundant 0.25 taxa. "mix" means 0.5 of the differential taxa come from "abundnat" and "rare", respectively.
#' @param covariate.type a character:"binary", "continuous". "binary" is default, means covariate is binary variable, i.e., case vs control. "continuous" means the covariate is continuous variable, i.e., age.
#' @param covariate.eff.mean a number: indicating the effect size, default is 0.
#' @param grp.ratio a number between 0-1: indicating the ratio of case and control. default is 1, means 50 percentage samples are case and  50 percentage  are control. formula is grp.ratio / (1 + grp.ratio)).
#' @param covariate.eff.sd a number: indicating the variation of the taxa. default is 0, default is 0.
#' @param confounder.type a character: consists "none", "binary", "continuous", "both". default is "none", means no confounding variable. binary" means confounder is 2-group variable, i.e., sex. "continuous" means a continuous variable, i.e., BMI. "both" means two confouders, including binary and continuous variable.
#' @param conf.cov.cor a number between 0-1: indicating the correlation between covariate and confounders, default is 0.6.
#' @param conf.diff.otu.pct a number between 0-1: indicating the proportion of taxa are influenced by confounders, default is 0.05.
#' @param conf.nondiff.otu.pct a number between 0-1: indicating the proportion of taxa are not influenced by confounders, default is 0.1.
#' @param confounder.eff.mean a number: effect size of the confounding effect, default is 0.
#' @param confounder.eff.sd a number: variation of the taxa that have confounding effect. default is 0.
#' @param depth.mu an integer: define the mean sequencing depth to be simulated, default is 10000.
#' @param depth.theta theta value in defining the variance 'mu + mu^2/theta' of the negative binomial distribution, default is 5.
#' @param depth.conf.factor a number: depth confounding factor, 0 means no depth confounding, default is 0
#' @return Return a list with the elements:
#' \item{otu.tab.sim}{simulated otu table}
#' \item{covariate}{simulated covariate}
#' \item{diff.otu.ind}{index of the differential taxa}
#' \item{otu.names}{the otu names of the simulated otu table}
#' \item{conf.otu.ind}{taxa with confounding effect}
#' @rdname SimulateSeq
#' @import dirmult
#' @import MASS
#' @import stats
#' @export

SimulateMSeq <- function (
		ref.otu.tab, nSam = 100, nOTU = 500,
		diff.otu.pct = 0.1, diff.otu.direct = c("balanced", "unbalanced"), diff.otu.mode = c("abundant", "rare", "mix"),
		covariate.type = c("binary", "continuous"), grp.ratio = 1, covariate.eff.mean = 1, covariate.eff.sd = 0,
		confounder.type = c("none", "binary", "continuous", "both"), conf.cov.cor = 0.6, 
		conf.diff.otu.pct = 0, conf.nondiff.otu.pct = 0.1, confounder.eff.mean = 0, confounder.eff.sd = 0,
		error.sd = 0, depth.mu = 10000, depth.theta = 5,  depth.conf.factor = 0
){
	diff.otu.direct <- match.arg(diff.otu.direct)
	diff.otu.mode <- match.arg(diff.otu.mode)
	covariate.type <- match.arg(covariate.type)
	confounder.type <- match.arg(confounder.type)
	ref.otu.tab0 <- ref.otu.tab
	model.paras <- EstPara(ref.otu.tab = ref.otu.tab) # full ref.otu.tab + estimated parameters
	
	sample.names <- colnames(model.paras$ref.otu.tab)
	ref.otu.tab <- model.paras$ref.otu.tab[(1 : (nOTU)),] # otu * sample
	dim(ref.otu.tab)
	idx.otu <- rownames(ref.otu.tab)
	idx.sample <- sample(sample.names, nSam)
	idx.nonsample <- colnames(ref.otu.tab)[!(colnames(ref.otu.tab) %in% idx.sample)]
	
	ref.otu.tab = ref.otu.tab[,idx.sample] # otu * sample, final selected OTU tab by User
	ref.otu.tab.unselect = ref.otu.tab0[c(1 : (nOTU)),][,idx.nonsample] # For overfitting test
	
	
	if (confounder.type == "none")  {
		confounder.type <- "continuous"
		confounder.eff.mean <- 0
		confounder.eff.sd <- 0
		Z <- NULL
	}
	if (confounder.type == "continuous") Z <- cbind(rnorm(nSam))
	if (confounder.type == "binary") Z <- cbind(c(rep(0, nSam %/% 2), rep(1, nSam - nSam %/% 2)))
	if (confounder.type == "both") Z <- cbind(rnorm(nSam), c(rep(0, nSam %/% 2), rep(1, nSam - nSam %/% 2)))
	
	rho <- sqrt(conf.cov.cor ^ 2 / (1 - conf.cov.cor ^ 2))
	
	if (covariate.type == 'continuous') {
		X <- rho * scale(scale(Z) %*% rep(1, ncol(Z)))  + rnorm(nSam)
	}
	
	if (covariate.type == "binary") {
		X <- rho * scale(scale(Z) %*% rep(1, ncol(Z)))  + rnorm(nSam)
		X <- cbind(ifelse(X <= quantile(X, grp.ratio / (1 + grp.ratio)), 0, 1))
	}
	rownames(X) <- colnames(ref.otu.tab)
	
	covariate.eff.mean1 = covariate.eff.mean
	covariate.eff.mean2 = covariate.eff.mean
	
	if (diff.otu.direct == "balanced") {
		if (diff.otu.mode == 'abundant'){
			eta.diff <- sample(c(rnorm(floor(nOTU / 2), mean = -covariate.eff.mean2, sd = covariate.eff.sd),
							rnorm(nOTU - floor(nOTU / 2), mean = covariate.eff.mean2, sd = covariate.eff.sd)))  %*% t(scale(X)) # multiple means * xb, eta = xb
		}else if(diff.otu.mode == 'rare'){
			eta.diff <- sample(c(rnorm(floor(nOTU / 2), mean = -covariate.eff.mean2, sd = covariate.eff.sd),
							rnorm(nOTU - floor(nOTU / 2), mean = covariate.eff.mean2, sd = covariate.eff.sd)))  %*% t(scale(X))
		}else{
			eta.diff <- c(sample(c(rnorm(floor(nOTU / 4), mean = -covariate.eff.mean1, sd = covariate.eff.sd),
									rnorm(floor(nOTU / 2) - floor(nOTU / 4), mean = covariate.eff.mean1, sd = covariate.eff.sd))),
					sample(c(rnorm(floor((nOTU -floor(nOTU / 2))/2), mean = -covariate.eff.mean2, sd = covariate.eff.sd),
									rnorm(nOTU -floor(nOTU / 2) - floor((nOTU -floor(nOTU / 2))/2), mean = covariate.eff.mean2, sd = covariate.eff.sd)))) %*% t(scale(X))
		}
	}
	
	
	if (diff.otu.direct == "unbalanced") {
		if (diff.otu.mode == 'abundant'){
			eta.diff <- rnorm(nOTU, mean = covariate.eff.mean2, sd = covariate.eff.sd) %*% t(scale(X)) # otu in 50% samples +, 50% sample -
		}else if(diff.otu.mode == 'rare'){
			eta.diff <- rnorm(nOTU, mean = covariate.eff.mean2, sd = covariate.eff.sd) %*% t(scale(X))
		}else{
			eta.diff <- c(sample(c(rnorm(floor(nOTU / 2), mean = covariate.eff.mean1, sd = covariate.eff.sd))),
					sample(c(rnorm(nOTU - floor(nOTU / 2), mean = covariate.eff.mean2, sd = covariate.eff.sd)))) %*% t(scale(X))
		}
	}
	
	eta.conf <-  sample(c(rnorm(floor(nOTU / 2), mean = -confounder.eff.mean, sd = confounder.eff.sd),
					rnorm(nOTU - floor(nOTU / 2), mean = confounder.eff.mean, sd = confounder.eff.sd)))  %*% t(scale(scale(Z) %*% rep(1, ncol(Z))))
	
	otu.ord <- 1 : (nOTU)
	diff.otu.ind <- NULL
	diff.otu.num <- round(diff.otu.pct * nOTU)
	
	if (diff.otu.mode == 'mix') diff.otu.ind <- c(diff.otu.ind, sample(otu.ord, diff.otu.num))
	if (diff.otu.mode == 'abundant') diff.otu.ind <- c(diff.otu.ind, sample(otu.ord[1 : round(length(otu.ord) / 4)], diff.otu.num))
	if (diff.otu.mode == 'rare') diff.otu.ind <- c(diff.otu.ind, sample(otu.ord[round(3 * length(otu.ord) / 4) : length(otu.ord)], diff.otu.num))
	
#	if(diff.otu.pct > 0){
#		conf.otu.ind <- c(sample(diff.otu.ind, round(nOTU * conf.diff.otu.pct)), sample(setdiff(1 : (nOTU), diff.otu.ind), round(conf.nondiff.otu.pct * nOTU)))
#		eta.diff[setdiff(1 : (nOTU), diff.otu.ind), ] <- 0 # some OTUs will be 0
#		eta.conf[setdiff(1 : (nOTU), conf.otu.ind), ] <- 0
#	}else{
#		eta.diff <- eta.diff
#		eta.conf <- eta.conf
#		conf.otu.ind <- NULL
#	}
	
    if (length(diff.otu.ind) >= round(nOTU * conf.diff.otu.pct)) {
		conf.otu.ind1 <- sample(diff.otu.ind, round(nOTU * conf.diff.otu.pct))
	} else {
		conf.otu.ind1 <- diff.otu.ind
	}
    conf.otu.ind <- c(conf.otu.ind1, sample(setdiff(1 : (nOTU), diff.otu.ind), round(conf.nondiff.otu.pct * nOTU)))
	
    eta.diff[setdiff(1 : (nOTU), diff.otu.ind), ] <- 0 # some OTUs will be 0
    eta.conf[setdiff(1 : (nOTU), conf.otu.ind), ] <- 0
	eta.error <- matrix(rnorm(nOTU * nSam, 0, error.sd), nOTU, nSam)

	
	eta.exp <- exp(t(eta.diff + eta.conf + eta.error))
	eta.exp <- eta.exp * t(ref.otu.tab)
	
	
	ref.otu.tab.prop <- eta.exp / rowSums(eta.exp)
	ref.otu.tab.prop <- t(ref.otu.tab.prop) # otu * sample
	
	nSeq <- rnegbin(nSam, mu = depth.mu * exp(scale(X) * depth.conf.factor), theta = depth.theta)

	otu.tab.sim <- sapply(1:ncol(ref.otu.tab.prop), function (i) rmultinom(1, nSeq[i], ref.otu.tab.prop[,i]))
	
	colnames(otu.tab.sim) <- rownames(eta.exp)
	rownames(otu.tab.sim) <- rownames(ref.otu.tab)
	
	
	diff.otu.ind = (1 : nOTU) %in% diff.otu.ind
	conf.otu.ind = (1 : nOTU) %in% conf.otu.ind
	
	return(list(otu.tab.sim = otu.tab.sim, covariate = X, confounder = Z, diff.otu.ind = diff.otu.ind, otu.names = idx.otu, conf.otu.ind = conf.otu.ind))
}


#' @title Simulate correlated microbiome data for differential abundance analysis
#' @param ref.otu.tab a matrix, the reference OTU count table (row - OTUs, column - samples), serving as the template for synthetic sample generation.
#' @param nSubject the number of subjects to be simulated.
#' @param nOTU the number of OTUs to be simulated.
#' @param nTime the number of time points to be simulated.
#' @param error.sd variance of the random error controls the within-subject correlation for taxa, default is 1.
#' @param MgX mean of the group effect across taxa.
#' @param SgX variance of the group effect across taxa.
#' @param X.diff.otu.pct a numeric value between 0 and 1, the percentage of differential OTUs for group effect to be simulated. If 0, global null setting is simulated. The default is 0.1.
#' @param grp.ratio a numeric value between 0 and 1. Group size ratio. The default is 1, means equal group size.
#' @param balanced.X a logical value "TRUE" or "FALSE" indicates the direction of change for these differential OTUs for group effect. "TRUE" means the direction of change for these differential OTUs is random. "FALSE" the direction of change is the same. The default is "TRUE".
#' @param MgT mean of the time effect across taxa. Default is 0.
#' @param SgT variance of the time effect across taxa. Default is 0.
#' @param SbT variance of the subject-specific time effect reflecting the between-subject variability. Default is 0.
#' @param T.diff.otu.pct a numeric value between 0 and 1, the percentage of differential OTUs for time effect to be simulated. If 0, global null setting is simulated. The default is 0.
#' @param balanced.T a logical value "TRUE" or "FALSE" indicates the direction of change for these differential OTUs for time effect. "TRUE" means the direction of change for these differential OTUs is random. "FALSE" the direction of change is the same. The default is "TRUE".
#' @param MgXT mean of the time effect across taxa. Default is 0.
#' @param SgXT variance of the time effect across taxa. Default is 0.
#' @param XT.diff.otu.pct a numeric value between 0 and 1, the percentage of differential OTUs for X:T interaction effect to be simulated. If 0, global null setting is simulated. The default is 0.
#' @param balanced.XT a logical value "TRUE" or "FALSE" indicates the direction of change for these differential OTUs for X:time interaction effect. "TRUE" means the direction of change for these differential OTUs is random. "FALSE" the direction of change is the same. The default is "TRUE".
#' @param MgZ mean of the confounder effect across taxa. Default is 0.5.
#' @param SgZ variance of confounder effect across taxa. Default is 0.
#' @param Z.diff.otu.pct a numeric value between 0 and 1, the percentage of differential OTUs for confounder effect to be sampled from group effect. If 0, global null setting is simulated. The default is 0.05.
#' @param Z.nondiff.otu.pct a numeric value between 0 and 1, the percentage of differential OTUs for confounder effect to be sampled without group effect. If 0, global null setting is simulated. The default is 0.1.
#' @param conf.cov.cor a numeric value between 0 and 1. The correlation between the covariate of interest and the confounder. The default is 0.6.
#' @param depth.mu the mean sequencing depth to be simulated. The default is 10,000.
#' @param depth.theta the theta value of the negative binomial distribution controlling the variance (mu + mu^2/theta). The default is 5.
#' @param depth.conf.factor a numeric value controlling the dependence of the sequencing depth on the covariate of interest (depth.mu * exp(scale(X) * depth.conf.factor)). The default is 0, i.e., the depth is not associated with the covariate of interest. This parameter can be used to simulate depth confounding.
#' @return Return a list with the elements:
#' \item{otu.tab.sim}{simulated OTU table}
#' \item{meta}{simulated meta data, including columns: covariate of interest(X), confounder(Z), time variable(time) and SubjectID}
#' \item{otu.names}{the names of the simulated OTUs}
#' \item{X.diff.otu.ind}{indices of the differential OTUs affected by the covariate of interest}
#' \item{T.diff.otu.ind}{indices of the differential OTUs affected by the time}
#' \item{XT.diff.otu.ind}{indices of the differential OTUs affected by the interaction between the covariate of interest and time}
#' \item{Z.diff.otu.ind}{indices of the differential OTUs affected by the confounder}
#' @rdname SimulateMSeqC
#' @import dirmult
#' @import MASS
#' @import stats
#' @export



SimulateMSeqC <- function (
		# Input data
		ref.otu.tab, # otu * sample
		nSubject = 40, nOTU = 50, nTime = 2,
		# Random error
		error.sd = 1, 
		# Group effect
		MgX = 0.5, SgX = 0, X.diff.otu.pct = 0.1, grp.ratio = 1, balanced.X = TRUE,
		# Time effect
		MgT = 0, SgT = 0, SbT = 0, T.diff.otu.pct =0, balanced.T = TRUE,
		# Interaction effect
		MgXT = 0, SgXT = 0, XT.diff.otu.pct = 0, balanced.XT = TRUE,
		# Confounding effect
		conf.cov.cor = 0.6, confounder = c('X', 'T'),
		MgZ = 0.5, SgZ = 0,  Z.diff.otu.pct = 0.05, Z.nondiff.otu.pct = 0.1, 
		# Sequence depth
		depth.mu = 10000, depth.theta = 5,  depth.conf.factor = 0
){
	confounder <- match.arg(confounder)
	if (confounder  == 'X') {
		if (Z.diff.otu.pct > X.diff.otu.pct)  stop("Z.diff.otu.pct should not be larger than X.diff.otu.pct!\n")
	}
	if (confounder  == 'T') {
		if (Z.diff.otu.pct > T.diff.otu.pct)  stop("Z.diff.otu.pct should not be larger than T.diff.otu.pct!\n")
	}
	
	## Estimate parameters
	model.paras <- EstPara(ref.otu.tab)
	## select top nOTU
	sample.names <- colnames(model.paras$ref.otu.tab)
	idx.sample <- sample(sample.names, nSubject, replace = T)
	OTU <- model.paras$ref.otu.tab[,idx.sample, drop=F]
	otu.names <- names(sort(rowMeans(OTU),decreasing=T))[1:nOTU]
	otu_tab <- OTU[otu.names,, drop=F]
	
	error.mean <- 0
	## Replicate template sample in each Subject, generate baseline otu table
	otu.tab <- otu_tab[, rep(1:nSubject, each=nTime)]
	colnames(otu.tab) <- paste0(paste0('rep',1:nTime), '_',rep(paste0('Subject',1:nSubject), each = nTime))
	
	## Generate meta data
	nSam <- ncol(otu.tab)
	SubjectID <- as.numeric(gsub('.*Subject','',colnames(otu.tab)))
	time <- as.numeric(as.factor(gsub('_.*','',colnames(otu.tab))))-1
	
	X <- as.vector(ifelse(SubjectID <= quantile(unique(SubjectID), grp.ratio / (1 + grp.ratio)), 0, 1))

	rho <- sqrt(conf.cov.cor ^ 2 / (1 - conf.cov.cor ^ 2))
	
	if (confounder == 'X') {
		Z <- as.vector(rho * scale(X) + rnorm(nSam))
	} else {
		Z <- as.vector(rho * scale(time) + rnorm(nSam))
	}
	
	meta <- cbind.data.frame(X, Z, SubjectID, time)
	rownames(meta) <- colnames(otu.tab)
	
	## Generate random error
	error <- replicate(nSam,rnorm(nOTU, error.mean, error.sd))
	
	## Generate diff.otu.ind
	otu.ord <- 1:nOTU
	X.diff.otu.ind <- T.diff.otu.ind <- XT.diff.otu.ind <-NULL
	
	X.diff.otu.num <- round(X.diff.otu.pct * nOTU)
	X.diff.otu.ind <- c(X.diff.otu.ind, sample(otu.ord, X.diff.otu.num))
	
	T.diff.otu.num <- round(T.diff.otu.pct * nOTU)
	T.diff.otu.ind <- c(T.diff.otu.ind, sample(otu.ord, T.diff.otu.num))
	
	Z.diff.otu.num <- round(Z.diff.otu.pct * nOTU)
	Z.nondiff.otu.num <- round(Z.nondiff.otu.pct * nOTU)
	
	if(confounder == 'T'){
		Z.diff.otu.ind <- c(sample(T.diff.otu.ind, Z.diff.otu.num), sample(setdiff(otu.ord, T.diff.otu.ind), Z.nondiff.otu.num))
	}else{
		Z.diff.otu.ind <- c(sample(X.diff.otu.ind, Z.diff.otu.num), sample(setdiff(otu.ord, X.diff.otu.ind), Z.nondiff.otu.num))
	}
	
	## Generate coefficient for X
	if(balanced.X){
		coef.X <- sample(c(rnorm(floor(nOTU / 2), mean = -MgX, sd = SgX), rnorm(nOTU - floor(nOTU / 2), mean = MgX, sd = SgX)))
	}else{
		coef.X <- rnorm(nOTU, MgX, SgX)
	}
	
	coef.X[setdiff(otu.ord, X.diff.otu.ind)] <- 0
	eta.X <- coef.X  %*% t(scale(X))
	
	## Generate coefficient for time
	if(balanced.T){
		coef.T <- sample(c(rnorm(floor(nOTU / 2), mean = -MgT, sd = SgT), rnorm(nOTU - floor(nOTU / 2), mean = MgT, sd = SgT)))
	}else{
		coef.T <- rnorm(nOTU, MgT, SgT)
	}
	coef.T[setdiff(otu.ord, T.diff.otu.ind)] <- 0
	coef.T.Subject <- replicate(nSubject, rnorm(nOTU, coef.T, SbT))
	coef.T <- matrix(data = apply(coef.T.Subject, 2, function(x) rep(x, nTime)), ncol = ncol(coef.T.Subject)*nTime)
	eta.T <- t(t(coef.T) * as.vector(scale(time)))
	
	
	## Generate coefficient for interaction
	if(balanced.XT){
		coef.XT <- sample(c(rnorm(floor(nOTU / 2), mean = -MgXT, sd = SgXT), rnorm(nOTU - floor(nOTU / 2), mean = MgXT, sd = SgXT)))
	}else{
		coef.XT  <- rnorm(nOTU, MgXT, SgXT)
	}
	coef.XT[setdiff(otu.ord, XT.diff.otu.ind)] <- 0
	coef.XT.Subject <- replicate(nSubject, rnorm(nOTU, coef.XT, 0))
	coef.XT <- matrix(data = apply(coef.XT.Subject, 2, function(x) rep(x, nTime)), ncol = ncol(coef.XT.Subject)*nTime)
	eta.XT <-  t(t(coef.XT) * as.vector(scale(time) *scale(X)))
	
	## Generate coefficient for Z, assume balanced change
	coef.Z <- sample(c(rnorm(floor(nOTU / 2), mean = -MgZ, sd = SgZ), rnorm(nOTU - floor(nOTU / 2), mean = MgZ, sd = SgZ)))
	coef.Z[setdiff(otu.ord, Z.diff.otu.ind)] <- 0
	eta.Z <- coef.Z %*% t(scale(Z))
	
	## combine index matrix, coefficient matrix and design matrix
	eta.exp <- exp(t(eta.X + eta.Z + eta.T + eta.XT + error))
	eta.exp <- eta.exp * t(otu.tab)
	
	# Renormalize
	otu.tab.prop <- t(eta.exp / rowSums(eta.exp))
	
	# Generate the sequence depth
	nSeq <- rnegbin(ncol(otu.tab.prop), mu = depth.mu * exp(scale(X) * depth.conf.factor), theta = depth.theta)
	otu.tab.sim <- sapply(1:ncol(otu.tab.prop), function (i) rmultinom(1, nSeq[i], otu.tab.prop[,i]))
	colnames(otu.tab.sim) <- rownames(eta.exp)
	rownames(otu.tab.sim) <- colnames(eta.exp)
	
	otu.names <- colnames(eta.exp)
	
	X.diff.otu.ind = (1 : nOTU) %in% X.diff.otu.ind
	T.diff.otu.ind = (1 : nOTU) %in% T.diff.otu.ind
	XT.diff.otu.ind = (1 : nOTU) %in% XT.diff.otu.ind
	Z.diff.otu.ind = (1 : nOTU) %in% Z.diff.otu.ind
	
	return(list(otu.tab.sim = otu.tab.sim, meta = meta, otu.names = otu.names,
					X.diff.otu.ind = X.diff.otu.ind, T.diff.otu.ind = T.diff.otu.ind, XT.diff.otu.ind = XT.diff.otu.ind, Z.diff.otu.ind = Z.diff.otu.ind))
}

dICC <- function (dist.mat, strata) {
	n <- length(strata)
	strata <- factor(strata)
	m <- nlevels(strata)
	ns <- table(strata)
	weights <- strata
	levels(weights) <- ns
	weights <- as.numeric(as.character(weights))
	
	dist.mat <- as.matrix(dist.mat^2)
	
	with.mask <- dist.mat[, ]
	with.mask[, ] <- 0
	for (ID in levels(strata)) {
		ind <- strata == ID
		with.mask[ind, ind] <- 1
	}
	btw.mask <- !with.mask
	btw.mask[upper.tri(btw.mask)] <- with.mask[upper.tri(with.mask)] <- 0
	diag(btw.mask) <- diag(with.mask) <- 0
	
	SSW <- sum(dist.mat * with.mask / weights)
	SST <- (sum(dist.mat * btw.mask) + sum(dist.mat * with.mask)) / n
	SSB <- SST - SSW
	
	R2 <- SSB / SST
	
	SE <- SSW / (n - m)
	SB <- (SST - (n-1) * SE) / (n - sum(ns^2) / n)
	
	ICC <- SB / (SB + SE)
	
	return(list(ICC = ICC))
}



# For N by M design only
dICC.SE.asympt <- function (dist.mat, strata) {
	
	strata <- factor(strata)
	N <- nlevels(strata)
	if (length(strata) %% N != 0) {
		stop("Asmptotic SE calculation only supports equal numbers of technical replicates! Consider using bootstrap SE instead!\n")
	}

	M <- length(strata) / N
	
	M.exp <- outer(1:M, 1:M, paste)
	M.exp <- M.exp[lower.tri(M.exp)]
	M.exp <- strsplit(M.exp, ' ')
	
	# first order the mat
	strata <- as.numeric(strata)
	obj <- sort(strata, index.return = TRUE)
	mat <- dist.mat[obj$ix, obj$ix]
	strata <- obj$x
	index <- rep(1:M, N)
	
	mat <- as.matrix(mat^2)
	
	# Calculate Tn and Gn
	TN <- 0
	for (i in 1:length(M.exp)) {
		index1 <- as.numeric(M.exp[[i]])
		TN <- TN + sum(mat[cbind(which(index == index1[1]), which(index == index1[2]))])
	}
	TN <- TN * 2 / N / M / (M - 1)
	
	GN <- 0
	for (i in 1:M) {
		for (j in 1:M) {
			mat1 <- mat[which(index == i), which(index == j)]
			diag(mat1) <- 0
			GN <- GN + sum(mat1)
		}
	}
	GN <- GN / N / (N - 1) / M^2
	
	# Calculate variance component s11
	s11 <- 0
	for (i in 1:length(M.exp)) {
		index1 <- as.numeric(M.exp[[i]])
		for (j in 1:length(M.exp)) {
			index2 <- as.numeric(M.exp[[j]])
			s11 <- s11 + mean(mat[cbind(which(index == index1[1]), which(index == index1[2]))] * 
							mat[cbind(which(index == index2[1]), which(index == index2[2]))]) - TN^2
		}
	}
	s11 <- 4 * s11 / M^2 / (M - 1)^2 
	
	# Calculate variance component s22
	s22 <- 0
	for (s in 1:M) {
		for (u in 1:M) {
			t1 <- 0
			t2 <- 0
			for (v in 1:M) {
				mat1 <- (mat[which(index == s), which(index == v)])
				mat2 <- (mat[which(index == u), which(index == v)]) 
				t1 <- t1 + (rowSums(mat1) - diag(mat1))
				t2 <- t2 + (rowSums(mat2) - diag(mat2))			
			}
			t1 <- t1 / (N - 1) / M
			t2 <- t2 / (N - 1) / M
			
			s22 <-  s22 +  mean(t1 * t2) - GN^2
		}
	}
	s22 <- 4  / M^2 * s22
	
	# Calculate variance component s12
	s12 <- 0
	for (i in 1:length(M.exp)) {
		index1 <- as.numeric(M.exp[[i]])
		vec <- mat[cbind(which(index == index1[1]), which(index == index1[2]))]
		for (s in 1:M) {
			t1 <- 0
			for (u in 1:M) {
				mat1 <- (mat[which(index == s), which(index == u)])
				t1 <- t1 + (rowSums(mat1) - diag(mat1))
			}
			t1 <- t1 / (N - 1) / M
			s12 <- s12 + mean(vec * t1) - TN * GN
		}
	}
	s12 <- 4 * s12 / M^2 / (M - 1)
	
	# http://www.stat.cmu.edu/~hseltman/files/ratio.pdf
	SE <- sqrt(TN^2 / GN^4 * s22 + s11 / GN^2 - 2 * TN / GN^3 * s12) / sqrt(N)
#	return(list(s11 = s11, s22 = s22, s12 = s12, SE = SE,
#					TN = TN, GN = GN, ICC = 1 - TN / GN))
	return(list(ICC = 1 - TN / GN, SE = SE))
}




dICC.SE.bt <- function (dist.mat, strata, B = 199) {
	
	paste2 <- function(x, collapse) {
		if (length(x) == 1) {
			return(paste(x))
		} else {
			return(paste(x, collapse=collapse))
		}
	}
	
# subject-based ID
	boot.ind <- function (ind, subject) {
		subject <- factor(subject)
		temp <- tapply(ind, subject, paste2, collapse = ' ')
		temp <- temp[sample(levels(subject), replace = TRUE)]
		strata <- paste0(names(temp), 1:length(temp))
		temp <- strsplit(temp, ' ')
		return(list(ind=as.numeric(unlist(temp)), strata=factor(rep(strata, sapply(temp, length)))))
	}
	
	ind <- 1:nrow(dist.mat)
	res <- sapply(1:B, function (i) {
				temp <- boot.ind(ind, strata)
				ind0 <- temp$ind
				strata0 <- temp$strata
				dICC(dist.mat[ind0, ind0], strata0)$ICC	
			})
	return(list(ICC = dICC(dist.mat, strata)$ICC, SE = sd(res)))
}

#
#rdirichlet1 <- function (n = 1, alpha) {
#	p <- length(alpha)
#	Gam <- matrix(rgamma(n * p, shape = alpha), ncol = p, byrow = TRUE)
#	Gam / rowSums(Gam)
#}
#
#rdirichlet2 <- function(Pi, t2) {
#	
#	n <- nrow(Pi)
#	p <- ncol(Pi)
#	alpha <- t(Pi * t2)
#	Gam <- matrix(rgamma(n * p, shape = alpha), ncol = p, byrow = TRUE)
#	Gam / rowSums(Gam)
#	
#}

perm.fdr.adj <- function (F0, Fp) {
	ord <- order(F0, decreasing = T)
	F0 <- F0[ord]
	perm.no <- ncol(Fp)
	Fp <- as.vector(Fp)
	Fp <- Fp[!is.na(Fp)]
	Fp <- sort(c(Fp, F0), decreasing = F)
	n <- length(Fp)
	m <- length(F0)
	FPN <- (n + 1) - match(F0, Fp) - 1:m
	p.adj.fdr <- FPN / perm.no / (1:m)
	
	# Impute 0s - pseudo-ct 0.5
	p.adj.fdr[p.adj.fdr == 0] <- 0.5 / perm.no / which(p.adj.fdr == 0)
	
	p.adj.fdr <- pmin(1, rev(cummin(rev(p.adj.fdr))))[order(ord)]

	return(p.adj.fdr)
}

perm.fwer.adj <- function (F0, Fp) {
	ord <- order(F0, decreasing = T)
	m <- length(F0)
	F0 <- F0[ord]
	Fp <- Fp[ord, , drop=FALSE]
	col.max <- Fp[m, ]
	p.adj.fwer <- sapply(m:1, function(i) {
				x <- F0[i]
				y <- Fp[i, ]
				col.max <<- ifelse(y > col.max, y, col.max)
				# Impute 0s
				mean(c(1, col.max >= x))
			})
	p.adj.fwer <- rev(p.adj.fwer)
	
	p.adj.fwer <- pmin(1, rev(cummin(rev(p.adj.fwer))))[order(ord)]
	return(p.adj.fwer)
}

na.pad <- function (vec, ind) {
	vec0 <- numeric(length(ind))
	vec0[!ind] <- vec
	vec0[ind] <- NA
	return(vec0)
}

find.ref.z1 <- function(otu.tab, M,  ref.pct = 0.50, feature.dat.type, p.max = 500) {
  
  # p.max to reduce the computation time
	p <- nrow(otu.tab)
	n <- ncol(otu.tab)
	
	if (p > p.max) {
	  ord <- order(rowSums(otu.tab), decreasing = TRUE)
	  otu.tab <- otu.tab[ord[1 : p.max], ]
	  p <- p.max
	} else {
	  ord <- 1 : p
	}

	if (feature.dat.type == 'count') {
		otu.tab <- otu.tab + 1
	}
	
	if (feature.dat.type == 'proportion') {
		otu.tab <- t(apply(otu.tab, 1, function (x) {
							x[x == 0] <- min(x[x != 0]) / 2
							return(x)
						}))
	}
	
	otu.tab <- t(otu.tab)
	res <- matrix(NA, p, p)
	
	XXI <- solve(t(M) %*% M)
	dof <- n - ncol(M)
	XXIM <- XXI %*% t(M)
	
	for (i in 1 : (p - 1)) {
		
		Y <- (as.matrix(log((otu.tab[, i]) / (otu.tab[, (i + 1) : p]))))

		est <- XXIM %*% Y
		resid <- Y - M %*% est
		sigma <- sqrt(colSums(resid^2) / dof)
		
		res[i, (i + 1) : p] <- res[(i + 1) : p, i] <- sigma
	}
	ref.ind <- order(rowMedians(res, na.rm = TRUE))[1 : round(ref.pct * p)]
	return(ord[ref.ind])
}

bbmix.fit.MM <- function (ct, dep,  nIter = 10, winsor.qt = 1.0) {
	
	if (mean(ct == 0) >= 0.95 | sum(ct != 0) < 5) {
		stop('The number of nonzero is too small to fit the model! Consider removing it from testing or please do not enable posterior sampling!\n')
	}
	
	# Initialization
	prop0 <- ct / dep
	qt <- quantile(prop0, winsor.qt)
	prop0[prop0 >= qt] <- qt
	
	var1 <- var(prop0) / 2
	mean1 <- mean(prop0) / 2
	
	var2 <- var(prop0) / 2
	mean2 <- 3 * mean(prop0) / 2
	
	pi <- 0.5
	
	for (i in 1 : nIter) {
		
		
		shape1.1 <- ((1 - mean1) / var1 - 1 / mean1) * mean1 ^ 2
		shape1.2 <- shape1.1 * (1 / mean1 - 1)
		
		shape2.1 <- ((1 - mean2) / var2 - 1 / mean2) * mean2 ^ 2
		shape2.2 <- shape2.1 * (1 / mean2 - 1)
		
		m1 <- shape1.1 / (shape1.1 + shape1.2)
		s1 <- shape1.1 + shape1.2
		
		m2 <- shape2.1 / (shape2.1 + shape2.2)
		s2 <- shape2.1 + shape2.2
		
		f1 <- dbetabinom(ct, dep, m1, s1)
		f2 <- dbetabinom(ct, dep, m2, s2)
		
		q1 <-  pi * f1 /  (pi * f1 + (1 - pi) * f2)
		q2 <-  1 - q1
		
		pi <- mean(q1)
		
		# Rough estimation
		mean1 <- sum(prop0 * q1) / sum(q1)
		var1 <- sum((prop0 - mean1)^2 * q1) / sum(q1)
		
		mean2 <- sum(prop0 * q2) / sum(q2)
		var2 <- sum((prop0 - mean2)^2 * q2) / sum(q2)
	}
	
	return(list(shape1.1 = shape1.1, shape1.2 = shape1.2, shape2.1 = shape2.1, shape2.2 = shape2.2, pi = pi, q1 = q1))
	
}

#' @title Permutation-based differential abundance analysis
#' @param meta.dat a data frame containing the sample information
#' @param feature.dat a matrix of counts, row - features (OTUs, genes, etc) , column - samples.
#' @param feature.dat.type  the type of the feature data. It could be "count", "proportion" or "other". For "proportion" data type, posterior sampling will not be performed,
#' but the reference-based ratio approach will still be used to address compositional effects. For "other" data type, neither posterior sampling or reference-base ratio approach
#' will be used.
#' @param grp.name the name for variable of interest. It could be numeric or categorical; should be in "meta.dat".
#' @param adj.name  the name for the variable(s) to be adjusted. They could be numeric or categorical; should be in "meta.dat."
#' @param prev.filter the prevalence cutoff, under which the features will  be filtered. 
#' @param mean.abund.filter the mean relative abundance cutoff, under which the features will  be filtered. 
#' @param max.abund.filter the max relative abundance cutoff, under which the features will  be filtered. 
#' @param min.prop  proportions less than this value will be replaced with this value. Only relevant when log transformation is used. The default is 0. 
#' @param is.winsor a logical value indicating whether winsorization should be performed to replace outliers. The default is TRUE.
#' @param winsor.end a character indicating whether the outliers at the top, bottom or both will be winsorized.
#' @param outlier.pct the percentage of expected outliers. These outliers will be winsorized.
#' @param is.post.sample a logical value indicating whether to perform posterior sampling of the underlying proportions. Only relevant when the feature data are counts.
#' @param post.sample.no the number of posterior samples if posterior sampling is used.
#' @param link.func  a list of transformation functions for the feautre data.
#' @param perm.no the number of permutations; If the raw p values are of the major interest, set "perm.no" to at least 999.
#' @param strata  a factor indicating the permutation strata; permutation will be confined to each stratum.
#' @param stats.combine.func function to combine the F-statistic for the omnibus test.
#' @param ref.pct percentage of reference taxa.
#' @param stage.no the number of stages if multiple-stage normalization is used.
#' @param excl.pct the maximum percentage of significant features in the reference set that can be removed. Only relevant when multiple-stage normalization is used.
#' @param p.max the maximum number of most abundant features to be considered in reference selection; only relevant for large data sets 
#' @param is.fwer a logical value indicating whether the family-wise error rate control (West-Young) will be performed.
#' @param verbose a logical value indicating whether the trace information should be printed out.
#' @param return.feature.dat a logical value indicating whether the wisorized, filtered "feature.dat" matrix should be returned.
#' @return A list with the elements
#' \item{call}{the call}
#' \item{feature.dat}{the wisorized, filtered "feature.dat" matrix}
#' \item{meta.dat}{"meta.dat" used}
#' \item{grp.name}{"grp.name" used}
#' \item{filter.ind}{a vector of logical values indicating which features are filtered}
#' \item{R2}{a matrix of percent explained variance (number of features by number of transformation functions)}
#' \item{F0}{a matrix of observed F-statistics (number of features by number of functions)}
#' \item{RSS}{a matrix of residual sum squares (number of features by number of functions)}
#' \item{df.model, df.residual}{degrees of freedom for the model and residual space}
#' \item{p.raw}{the raw p-values based on permutations (not accurate if "perm.no" is small)}
#' \item{p.adj.fdr}{permutation-based FDR-adjusted p-values}
#' \item{p.adj.fwer}{permutation-based FWER-adjusted (West-Young) p-values}
#' @import stats
#' @import nlme
#' @import matrixStats
#' @import vegan
#' @import ape
#' @import statmod
#' @importFrom rmutil dbetabinom
#' @rdname ZicoSeq
#' @export

ZicoSeq <- function (
		meta.dat, feature.dat, grp.name, adj.name = NULL, feature.dat.type = c('count', 'proportion', 'other'),
		prev.filter = 0, mean.abund.filter = 0, max.abund.filter = 0, min.prop = 0,
		is.winsor = TRUE, outlier.pct = 0.03, winsor.end = c('top', 'bottom', 'both'),
		is.post.sample = TRUE,  post.sample.no = 25,
		link.func = list(function (x) sign(x) * (abs(x))^0.5), stats.combine.func = max,
		perm.no = 99,  strata = NULL, 
		ref.pct = 0.50, stage.no = 6, excl.pct = 0.20, p.max = 500,
		is.fwer = FALSE, verbose = TRUE, return.feature.dat = TRUE) {
	
	this.call <- match.call()
	feature.dat.type <- match.arg(feature.dat.type)
	winsor.end <- match.arg(winsor.end)
	
	if (!is.null(strata)) {
	  if (length(strata) == 1 & is.character(strata)) {
	    strata <- factor(meta.dat[, strata])
	  } else {
	    strata <- factor(strata)
	  }
	  if (length(strata) != nrow(meta.dat)) {
	    stop("'strata' parameter is not correct. Dimension does not match!\n")
	  }
	}
	
	if(is.null(rownames(feature.dat)[1])) {
		rownames(feature.dat) <- paste0("S", 1 : nrow(feature.dat))
	}
	
	sds <- rowSds(feature.dat)
	if (sum(sds == 0) != 0) {
		stop(paste('Feature ',  paste(which(sds == 0), collapse = ','), 
						'have identical values (e.g. all 0s)! Please remove them!\n'))
	}
	

	if (perm.no < 99) {
		warning('To obtain stable results, number of permutations should be at least 99!\n')
	}
	
	if (feature.dat.type == 'count') {
		if (sum(feature.dat < 0 | (feature.dat < 1 & feature.dat > 0)) != 0) {
			stop('It seems the feature data are not counts. Please check!\n')
		}
	}
	
	if (feature.dat.type == 'proportion') {
		if (sum(feature.dat < 0 | feature.dat > 1) != 0) {
			stop('It seems the feature data are not proportions. Please check!\n')
		}
	}
	
	if (feature.dat.type %in% c('proportion', 'other') & is.post.sample == TRUE) {
		cat('For proportion and other data types,  posterior sampling will not be performed!\n')
		is.post.sample <- FALSE
	}
	
	if (feature.dat.type == 'other') {
		stage.no <- 1
	}
	if (feature.dat.type == 'count' & nrow(meta.dat) <= 40) {
		cat('For sample size less than 40, posterior sampling will not be used!\n')
		is.post.sample <- FALSE
	}
	
	if (feature.dat.type == 'count' & is.post.sample == FALSE){
		if (min(coef(summary(lm(sqrt(colSums(feature.dat)) ~  meta.dat[, grp.name])))[-1, 'Pr(>|t|)']) < 0.05) {
			warning('The sequencing depth is correlated with the variable of interest!
							Rarefaction is needed to control for false positves!\n')
			
		}
	}

	###############################################################################
	# Filter to remove very rare taxa
	if (feature.dat.type %in% c('count', 'proportion')) {

		temp <- t(t(feature.dat) / colSums(feature.dat))

		filter.ind <- rowMeans(temp != 0) >= prev.filter & rowMeans(temp) >= mean.abund.filter & rowMaxs(temp) >= max.abund.filter
		names(filter.ind) <- rownames(feature.dat)
		if (verbose)  cat(sum(!filter.ind), ' features are filtered!\n')
		filter.features <- rownames(feature.dat)[!filter.ind]
		feature.dat <- feature.dat[filter.ind, ]
	} else {
		filter.ind <- rep(TRUE, ncol(feature.dat))
		filter.features <- NULL
	}
	
	# After filtering, some samples could have all zeros, will need to remove them.
	if (feature.dat.type %in% c('count', 'proportion')) {

	  ind <- colSums(feature.dat) != 0
	  
	  if (sum(!ind) != 0) {
	    warning(sum(!ind), ' samples have all zeros after filtering! They will be removed! \n')
	    feature.dat <- feature.dat[, ind, drop = FALSE]
	    meta.dat <- meta.dat[ind, ]
	    if (!is.null(strata)) {
	      strata <- strata[ind]
	    }
	  }

	}

	  
	if (feature.dat.type == 'proportion') {
		# Renormalization
		feature.dat <- t(t(feature.dat) / colSums(feature.dat))
	}
	
	sample.no <- ncol(feature.dat)
	otu.no <- nrow(feature.dat)
	row.names <- rownames(feature.dat)
	depth <- colSums(feature.dat)
	
	if (verbose) cat('The data has ', sample.no, ' samples and ', otu.no, ' features will be tested!\n' )

	if (feature.dat.type %in% c('count', 'proportion') & sum(rowSums(feature.dat != 0) <= 2) != 0) {
		warning('Some features have less than 3 nonzero values! 
               They have virtually no statistical power. Please consider removing them in the analysis!\n')
	}
	
	###############################################################################
	# Winsorization to reduce the influence of outliers
	
	if (is.winsor == TRUE) {
		depth <- colSums(feature.dat)
		
		cat("On average, ", round(length(depth) * outlier.pct), 
		                          " outlier counts will be replaced for each feature!\n") 

		feature.dat <- apply(feature.dat, 1, function (x) {

					if (feature.dat.type == 'count') {
						x <- x / depth
					}
					
					if (winsor.end == 'top') {
						qt <- quantile(x, 1 - outlier.pct)
						x[x >= qt] <- qt
					}
					
					if (winsor.end == 'bottom') {
						qt <- quantile(x, outlier.pct)
						x[x <= qt] <- qt
					}
					
					if (winsor.end == 'both') {
						qt1 <- quantile(x, 1 - outlier.pct / 2)
						qt2 <- quantile(x, outlier.pct / 2)
						x[x >= qt1] <- qt1
						x[x <= qt2] <- qt2
					}
					
					if (feature.dat.type == 'count') {
						return(round(x * depth))
					} else {
						return(x)
					}				
				})
		feature.dat <- t(feature.dat)
		sds <- rowSds(feature.dat)
		if (sum(sds == 0) != 0) {
			stop(paste('After winsorization, feature ',  paste(which(sds == 0), 
			             collapse = ','),  'have identical values! Did you set the "outlier.pct" too high or some features are extremely sparse?\n'))
		}
		
	}
	
	if (feature.dat.type == 'proportion') {
		# Renormalization
		feature.dat <- t(t(feature.dat) / colSums(feature.dat))
	}
	
	###############################################################################
	# Generate samples from posterior distribution (stacking it)
	if (is.post.sample == TRUE) {
		
		if (verbose) cat('Fitting beta mixture ...\n')
		feature.dat.p <- apply(feature.dat, 1, function (x) {
					err1 <- try(res <- bbmix.fit.MM(x, depth), silent = TRUE)
					
					# Handle error
					if (!inherits(err1, 'try-error')) {
						prop1.1 <- rbeta(sample.no * post.sample.no, shape1 = x + res$shape1.1, shape2 = res$shape1.2 + depth - x)
						prop1.2 <- rbeta(sample.no * post.sample.no, shape1 = x + res$shape2.1, shape2 = res$shape2.2 + depth - x)
						prop <- ifelse(runif(sample.no * post.sample.no) <= res$q1, prop1.1, prop1.2)
					} else {
						prop <- x / depth
						v <- var(prop)
						m <- mean(prop)
						
						a1 <- ((1 - m) / v - 1 / m) * m ^ 2
						a2 <- a1 * (1 / m - 1)
						
						if (is.na(a1) | a1 < 0) {
							# uniform prior
							prop <- rbeta(sample.no * post.sample.no, shape1 = x + 1, shape2 = otu.no + depth - x)
						} else {
							# beta prior
							prop <- rbeta(sample.no * post.sample.no, shape1 = x + a1, shape2 = a2 + depth - x)
						}
					}
					return(prop)
				})
		
		feature.dat.p <- t(feature.dat.p)
		feature.dat.p.list <- list()
		st <- 1
		end <- sample.no
		for (i in 1 : post.sample.no) {
			feature.dat.p.list[[i]] <- feature.dat.p[, st : end]
			# Normalization
			#    feature.dat.p.list[[i]] <- t(t(feature.dat.p[, st : end]) / colSums(feature.dat.p[, st : end]))
			st <- st + sample.no
			end <- end + sample.no
		}
	} else {
		feature.dat.p.list <- list()
		if (feature.dat.type == 'count') {
			feature.dat.p.list[[1]] <- t(t(feature.dat) / depth)
		} else{
			feature.dat.p.list[[1]] <- feature.dat
		}
		
		post.sample.no <- 1
	}
	
	# Replace zeros or extremely small values for log calculation
	if (feature.dat.type %in% c('count', 'proportion')) {
		for (i in 1 : post.sample.no) {
			temp <- feature.dat.p.list[[i]]
			temp[temp <= min.prop] <- min.prop
			feature.dat.p.list[[i]] <- temp
		}
	}
	#return(feature.dat.p.list)
	###############################################################################
	# Covariate space (including intercept)

	
	if (is.null(adj.name)) {
		M0 <- model.matrix(~ 1, meta.dat)
	} else {
		data0 <- meta.dat[, c(adj.name), drop = FALSE]
		if (sum(is.na(data0)) != 0) {
		  stop("Please remove or impute NAs in the variables to be adjusted!\n")
		}
		M0 <- model.matrix( ~ ., data0)
	}
	
	data1 <- meta.dat[, c(grp.name), drop = FALSE]
	
	if(sum(is.na(data1)) != 0) {
	  stop("Please remove or impute NAs in the variable of interest!\n")
	}
	
	M1 <-  model.matrix( ~ ., data1)[, -1, drop = FALSE]  # No intercept
	
	M01 <- cbind(M0, M1)
	
	# QR decompostion
	qrX0 <- qr(M0, tol = 1e-07)
	Q0 <- qr.Q(qrX0)
	Q0 <- Q0[, 1:qrX0$rank, drop = FALSE]
	H0 <- (Q0 %*% t(Q0))
	
	qrX1 <- qr(M1, tol = 1e-07)
	Q1 <- qr.Q(qrX1)
	Q1 <- Q1[, 1:qrX1$rank, drop = FALSE]
	
	qrX01 <- qr(M01, tol = 1e-07)
	Q01 <- qr.Q(qrX01)
	Q01 <- Q01[, 1:qrX01$rank, drop = FALSE]
	
	R0 <- as.matrix(resid(lm(Q1 ~ Q0 - 1)))
	
	pX0 <- ncol(Q0)
	pX1 <- ncol(Q1)
	pX01 <- ncol(Q01)
	
	df.model <- pX01 - pX0
	df.residual <- sample.no - pX01
	
	func.no <- length(link.func)
	
	###############################################################################
	if (feature.dat.type %in% c('count', 'proportion')) {
		if (verbose)  cat('Finding the references ...\n')
		ref.ind <- find.ref.z1(feature.dat, M0, ref.pct = ref.pct, feature.dat.type = feature.dat.type, p.max = p.max)
		size.factor <- colSums(feature.dat[ref.ind, ])
	} else {
		ref.ind <- NULL
	}
	###############################################################################
	# Perform multiple stage normalization
	if (verbose) cat('Permutation testing ...\n')
	# norm.ind <- NULL
	
	for (i in 1 : stage.no) {
		# Reference proportion
		
		if (feature.dat.type == 'other') {
			divisor <- 1
		} else {
			divisor <- size.factor / depth
			divisor[divisor == 0] <- 0.5 * min(divisor[divisor != 0])
		}
		
		# Create the giant Y matrix
		Y <- matrix(NA, sample.no, func.no * otu.no * post.sample.no)
		
		# Change order - func.no * otu.no
		for (k in 1 : post.sample.no) {
			for (j in 1 : func.no) {
				func <- link.func[[j]]
				feature.dat.p <- feature.dat.p.list[[k]]
				
				Y[, (k - 1) * func.no * otu.no + func.no * (0 : (otu.no - 1)) + j] <-
						func(t(feature.dat.p) / divisor)  # No scaling first
				
			}
		}
		
		Y <- t(Y)
		TSS <- rowSums(Y^2)
		MSS01 <- rowSums((Y %*% Q01)^2)
		MSS0 <- rowSums((Y %*% Q0)^2)
		
		MSS <- (MSS01 - MSS0)
		RSS <- (TSS - MSS01)
		
		
		getPermuteMatrix <- getFromNamespace("getPermuteMatrix", "vegan")
		perm.ind <- getPermuteMatrix(perm.no, sample.no, strata = strata)
		perm.no <- nrow(perm.ind)
		
		MRSSp <- sapply(1 : perm.no, function(ii) {
					if (verbose) {
						if (ii %% 10 == 0) cat('.')
					}
					
					Rp <- R0[perm.ind[ii, ], , drop = FALSE]
					# Project to the reisdual space
					Rp <- Rp - H0 %*% Rp
					
					qrRp <- qr(Rp, tol = 1e-07)
					Q1p <- qr.Q(qrRp)
					Q1p <- Q1p[, 1:qrRp$rank, drop = FALSE]
					
					MSS01p <- MSS0 + rowSums((Y %*% Q1p)^2)
					
					MSSp <- (MSS01p - MSS0)
					RSSp <- (TSS - MSS01p)
					
					c(MSSp, RSSp)
					
				})
		
		unit <- func.no * otu.no * post.sample.no
		MSSp <- MRSSp[1 : unit, ]
		RSSp <- MRSSp[(unit + 1) : (2 * unit), ]
		
		# EB is based on the aggregated RSS
		RSS.m <- array(RSS, c(func.no, otu.no,  post.sample.no))
		RSS.m <- t(apply(RSS.m, c(1, 2), mean))  # otu.no by func.no
		
		F0.m <- array((MSS / df.model) / (RSS / df.residual), c(func.no, otu.no,  post.sample.no))
		F0.m <-  t(apply(F0.m, c(1, 2), mean))  # otu.no by func.no
		
		R2.m <- array(MSS / TSS, c(func.no, otu.no, post.sample.no))
		R2.m <- t(apply(R2.m, c(1, 2), mean))  # otu.no by func.no
		
		F0 <- (MSS / df.model)  /  (RSS  / df.residual)
		Fp <- (MSSp / df.model)  /  (RSSp  / df.residual)
		
		# Expectation of F0 and Fp
		F0 <- array(F0, c(func.no, otu.no, post.sample.no))
		Fp <- array(Fp, c(func.no, otu.no, post.sample.no, perm.no))
		
		F0 <- apply(F0, c(1, 2), mean)    # func.no * otu.no
		Fp <- apply(Fp, c(1, 2, 4), mean) # func.no * otu.no * perm.no
		
		###############################################################################
		# Omnibus test by taking maximum
		F0 <- apply(F0, 2, stats.combine.func)
		Fp <- apply(Fp, c(2, 3), stats.combine.func)  # otu.no by perm.no
		
		if (verbose) cat('\n')
		
		if (mean(is.na(F0)) >= 0.1) {
			warning('More than 10% observed F stats have NA! Please check! \n')
		}
		
		if (mean(is.na(Fp)) >= 0.1) {
			warning('More than 10% permuted F stats have NA! Please check! \n')
		}
		
		na.ind <- is.na(F0)
		F0 <- F0[!na.ind]
		Fp <- Fp[!na.ind, ]
		
		which.nan.ind <- which(!na.ind)
		###############################################################################
		p.raw <- rowMeans(cbind(Fp, F0) >= F0)
		
		if (i == stage.no) {
			break
		} else {
			if (mean(p.raw <= 0.05) >= excl.pct) {
				ind <- order(p.raw)[1 : round(length(p.raw) * excl.pct)]
			} else {
				ind <- which(p.raw <= 0.05)
			}
			size.factor <- colSums(feature.dat[setdiff(ref.ind, which.nan.ind[ind]), ])
			# norm.ind <- cbind(norm.ind, !(1:nrow(feature.dat) %in% setdiff(ref.ind, which.nan.ind[ind])))
		}
	}
	
	
	
	p.adj.fdr <- perm.fdr.adj(F0, Fp)
	p.raw <- na.pad(p.raw, na.ind)
	p.adj.fdr <- na.pad(p.adj.fdr, na.ind)
	names(p.raw) <- names(p.adj.fdr) <- rownames(R2.m) <- rownames(RSS.m) <- rownames(F0.m) <- row.names
	colnames(R2.m) <- colnames(F0.m) <- colnames(RSS.m) <- paste0('Func', 1 : func.no)
	
	if (is.fwer) {
		p.adj.fwer <- perm.fwer.adj(F0, Fp)
		p.adj.fwer <- na.pad(p.adj.fwer, na.ind)
		names(p.adj.fwer)  <- row.names
	} else {
		p.adj.fwer <- NULL
	}
	
	# Finally, get the direction of change under specific transformations
    if (length(ref.ind) != 0) {
    divisor <- colSums(feature.dat[ref.ind, ])
    divisor[divisor == 0] <- 1
		temp.dat <- t(t(feature.dat) / divisor)
	} else {
		temp.dat <- feature.dat
	}
	
	coef.list <- NULL
	for (j in 1 : func.no) {
		func <- link.func[[j]]
		coef.list[[j]] <- solve(t(M01) %*% M01) %*% t(M01) %*% t(func(temp.dat))

	}

    
	if (!return.feature.dat) {
		feature.dat <- NULL
	}
	if (verbose) cat('Completed!\n')
	
	return(list(call = this.call, feature.dat = feature.dat, meta.dat = meta.dat, grp.name = grp.name,
	            filter.features = filter.features, ref.features = row.names[ref.ind],
					R2 = R2.m, F0 = F0.m, RSS = RSS.m, df.model = df.model, df.residual = df.residual, coef.list = coef.list,
					p.raw = p.raw, p.adj.fdr = p.adj.fdr,  p.adj.fwer = p.adj.fwer))
	
}



#' Plot ZicoSeq results
#' The function plots the association strength and direction based on the output from \code{ZicoSeq}.
#' @param ZicoSeq.obj return from function \code{ZicoSeq}.
#' @param pvalue.type character; It could be 'p.adj.fdr','p.raw' or 'p.adj.fwer'.
#' @param cutoff a real value between 0 and 1; cutoff for pvalue.type.
#' @param text.size text size for the plots.
#' @param out.dir character; the directory to save the figures, e.g., \code{getwd()}. Default is NULL. If NULL, figures will not be saved.
#' @param file.name character; the name of the file.
#' @param width the width of the graphics region in inches. See R function \code{ggsave}.
#' @param height the height of the graphics region in inches. See R function \code{ggsave}.
#' @return A \code{ggplot2} object.
#' @import ggplot2
#' @import dplyr
#' @import tibble
#' @importFrom ggpubr ggarrange
#' @rdname ZicoSeq.plot
#' @export
#' 


ZicoSeq.plot <- function(ZicoSeq.obj, pvalue.type = c('p.adj.fdr','p.raw','p.adj.fwer'), 
		cutoff = 0.1, text.size = 10, out.dir = NULL, file.name = 'ZicoSeq.plot.pdf',  width = 10, height = 6){
	
#  if(sum(ZicoSeq.obj[[pvalue.type]]<= cutoff) == 0){
#    cat(paste0('No taxa within ', pvalue.type, ' <= ',cutoff, '!'))
#  }
	
	grp.name <-  ZicoSeq.obj$grp.name
	meta.dat <- ZicoSeq.obj$meta.dat
	
	# cat('Significant(',pvalue.type,'<=',cutoff,') association strength between taxa and ',grp.name, ' is visualized!\n' )
	
	if(ZicoSeq.obj$call$feature.dat.type == 'proportion'){
		abundance <- ZicoSeq.obj$feature.dat
		prevalence <- apply(ZicoSeq.obj$feature.dat, 1, function(x) mean(x > 0))
	}
	
	if(ZicoSeq.obj$call$feature.dat.type == 'count'){
		abundance <- t(t(ZicoSeq.obj$feature.dat)/colSums(ZicoSeq.obj$feature.dat))
		prevalence <- apply(ZicoSeq.obj$feature.dat, 1, function(x) mean(x > 0))
	}
	
	if(ZicoSeq.obj$call$feature.dat.type == 'other'){
		abundance <- ZicoSeq.obj$feature.dat
		prevalence <- apply(abundance, 1, function(x) sd(x))
	}
	
	# relative abundance
	abundance <- apply(abundance, 1, function(x) mean(x))
	
	if(!(length(unique(meta.dat[,grp.name])) > 2 & !is.numeric(meta.dat[,grp.name]))) {
		coefs <- sapply(ZicoSeq.obj$coef.list, function(x) x[grep(grp.name,rownames(x)),])
		colnames(coefs) <- paste0('coef_Func',1:length(ZicoSeq.obj$coef.list))
	}  
	
	if (!is.numeric(meta.dat[,grp.name]) & length(unique(meta.dat[,grp.name])) == 2) {
		level2 <- rownames(ZicoSeq.obj$coef.list[[1]])[grep(paste0('^',grp.name),rownames(ZicoSeq.obj$coef.list[[1]]))]
		level2 <- gsub(grp.name, '', level2)
		base <- setdiff(unique(meta.dat[,grp.name]), level2)
	}
	
	# Break the ties
	ZicoSeq.obj$R2 <- ZicoSeq.obj$R2 + runif(length(ZicoSeq.obj$R2), 0, 1e-10)
	
	R2 <- rowMaxs(ZicoSeq.obj$R2)

	## pvalue = 0??? -log10(p)
	if(length(unique(meta.dat[,grp.name])) > 2 & !is.numeric(meta.dat[,grp.name])){
		plot.data <- data.frame(pvals = ZicoSeq.obj[[pvalue.type]], 
				prevalence = prevalence, abundance = abundance, 
				R2 = R2, 
				taxa = rownames(ZicoSeq.obj$R2))
		plot.data[plot.data$pvals > cutoff, 'taxa'] <- ''
		
		
	}else{

		signs <- t(sign(coefs))[t(ZicoSeq.obj$R2 == R2)]
		signs[signs == 0] <- 1
		
		plot.data <- data.frame(pvals = ZicoSeq.obj[[pvalue.type]], 
				prevalence = prevalence, abundance = abundance, 
				R2 = R2 * signs, 
				taxa = rownames(ZicoSeq.obj$R2))
		
		plot.data[plot.data$pvals > cutoff, 'taxa'] <- ''
		
		
	}
	
	if(is.numeric(meta.dat[,grp.name])) {
		title.name = paste0('Association between ', grp.name, ' and  taxa abundance')
	}
	
	if(!is.numeric(meta.dat[,grp.name]) & length(unique(meta.dat[,grp.name])) > 2){
		title.name = paste0('Association between ', grp.name, ' and  taxa abundance')
	}
	
	if(!is.numeric(meta.dat[,grp.name]) & length(unique(meta.dat[,grp.name])) == 2){
		title.name = paste0('Differential abundance between ', paste0(base, ' (reference) and ', level2))
	}
	
	pvals <- R2 <- taxa <- NULL
	plot.obj <-
			ggplot(plot.data, aes(x = R2, y = -log10(pvals))) +
			geom_point(aes(size = abundance, color = prevalence)) +
			geom_vline(aes(xintercept = 0), color = 'gray', linetype = 'dashed') +
			geom_hline(aes(yintercept = -log10(cutoff)), color = 'gray', linetype = 'dashed') +
			scale_colour_gradient2(low = "white", high = "#006D2C") +
			scale_y_continuous(limits = c(0, max(-log10(plot.data$pvals)) * 1.3)) +
			ggrepel::geom_text_repel(aes(label = taxa), max.overlaps = Inf, color = 'black') +
			labs(x = bquote(R^2), y = paste0('-log10(',pvalue.type,')'),
					color = ifelse(ZicoSeq.obj$call$feature.dat.type == 'other','Standard deviation','Prevalence'),
					size = ifelse(ZicoSeq.obj$call$feature.dat.type == 'other','Mean value','Mean abundance')) +
			theme_bw() +
			theme(axis.text = element_text(color = 'black', size = text.size),
					axis.title = element_text(color = 'black', size = text.size),
					legend.text = element_text(color = 'black', size = text.size),
					legend.title = element_text(color = 'black', size = text.size)) +
			ggtitle(title.name)
	
	
	
#	plots <- ggarrange(plotlist = plot.list, common.legend = T)
	
	if(!is.null(out.dir)) {
		print(plot.obj)
		ggsave(paste0(out.dir, file.name), width = width, height = height)
	}
	return(plot.obj)
}

GUniFrac <- function (otu.tab, tree, size.factor = NULL, alpha = c(0, 0.5, 1), verbose = TRUE) {
  # Calculate Generalized UniFrac distances. Unweighted and 
  # Variance-adjusted UniFrac distances will also be returned.
  #	
  # Args:
  #		otu.tab: OTU count table, row - n sample, column - q OTU
  #		tree: rooted phylogenetic tree of R class "phylo"
  #       size.factor:  the normalizing factor, which will divide the counts,
  #			 		 if not supplied, we will use total sum
  #		alpha: parameter controlling weight on abundant lineages
  #
  # Returns:
  # 	unifracs: three dimensional array containing the generalized 
  #							UniFrac distances, unweighted UniFrac distances
  #
  
  if (!is.rooted(tree)) stop("Rooted phylogenetic tree required!")
  
  # Convert into normalized data
  otu.tab <- as.matrix(otu.tab)
  if (is.null(size.factor)) {
    row.sum <- rowSums(otu.tab)
  } else {
    if (length(size.factor) != nrow(otu.tab)) {
      stop('The length of size factor is not consistent with the OTU table!\n')
    }	
    row.sum <- size.factor
  }
  row.sum <- rowSums(otu.tab)
  otu.tab <- otu.tab / row.sum
  n <- nrow(otu.tab)
  
  # Construct the returning array
  if (is.null(rownames(otu.tab))) {
    rownames(otu.tab) <- paste("comm", 1:n, sep="_")
  }
  # d_UW: unweighted UniFrac, d_VAW: weighted UniFrac
  dimname3 <- c(paste("d", alpha, sep="_"), "d_UW")
  unifracs <- array(NA, c(n, n, length(alpha) + 1),
                    dimnames=list(rownames(otu.tab), rownames(otu.tab), dimname3))
  for (i in 1:(length(alpha) + 1)){
    for (j in 1:n){
      unifracs[j, j, i] <- 0
    }
  }	
  
  # Check OTU name consistency
  if (sum(!(colnames(otu.tab) %in% tree$tip.label)) != 0) {
    stop("The OTU table contains unknown OTUs! OTU names
						in the OTU table and the tree should match!" )
  }
  
  # Get the subtree if tree contains more OTUs
  absent <- tree$tip.label[!(tree$tip.label %in% colnames(otu.tab))]
  if (length(absent) != 0) {
    tree <- drop.tip(tree, absent)
    warning("The tree has more OTU than the OTU table!")
  }
  
  # Reorder the otu.tab matrix if the OTU orders are different
  tip.label <- tree$tip.label
  otu.tab <- otu.tab[, tip.label]
  
  ntip <- length(tip.label)
  nbr <- nrow(tree$edge)	
  edge <- tree$edge
  edge2 <- edge[, 2]
  br.len <- tree$edge.length
  
  if(verbose)
  cat("Accumulate the abundance along the tree branches...\n")
  #  Accumulate OTU proportions up the tree	
  cum <- matrix(0, nbr, n)							# Branch abundance matrix
  for (i in 1:ntip) {
    tip.loc <- which(edge2 == i)
    cum[tip.loc, ] <- cum[tip.loc, ] + otu.tab[, i]	
    node <- edge[tip.loc, 1]						# Assume the direction of edge 
    node.loc <- which(edge2 == node)
    while (length(node.loc)) {
      cum[node.loc, ] <- cum[node.loc, ] + otu.tab[, i]		
      node <- edge[node.loc, 1]
      node.loc <- which(edge2 == node)
    }
  }
  
  if(verbose)
  cat("Compute pairwise distances ...\n")
  # Calculate various UniFrac distances
  #	cum.ct <- round(t(t(cum) * row.sum)) 	# For VAW
  #	for (i in 2:n) {
  #		for (j in 1:(i-1)) {
  #			cum1 <- cum[, i]
  #			cum2 <- cum[, j]
  #			ind <- (cum1 + cum2) != 0
  #			cum1 <- cum1[ind]
  #			cum2 <- cum2[ind]
  #			br.len2 <- br.len[ind]			
  #			mi <- cum.ct[ind, i] + cum.ct[ind, j]
  #			mt <- row.sum[i] + row.sum[j]			
  #			diff <- abs(cum1 - cum2) / (cum1 + cum2)	
  #			
  #			# Generalized UniFrac distance
  #			for (k in 1:length(alpha)) {
  #				w <- br.len2 * (cum1 + cum2) ^ alpha[k]
  #				unifracs[i, j, k] <- unifracs[j, i, k] <- sum(diff * w) / sum(w)
  #			}
  #			#	Variance Adjusted UniFrac Distance
  #			ind2 <- (mt != mi)
  #			w <- br.len2 * (cum1 + cum2) / sqrt(mi * (mt - mi))
  #			unifracs[i, j, (k + 2)] <- unifracs[j, i, (k + 2)] <- 
  #					sum(diff[ind2] * w[ind2]) / sum(w[ind2])		
  #			
  #			#	Unweighted UniFrac Distance
  #			cum1 <- (cum1 != 0)
  #			cum2 <- (cum2 != 0)			
  #			unifracs[i, j, (k + 1)] <- unifracs[j, i, (k + 1)] <- 
  #					sum(abs(cum1 - cum2) / (cum1 + cum2) * br.len2) / sum(br.len2)
  #		}
  #	}
  unifracs[, , ] <-  PairwiseD(cum, br.len, alpha)$unifracs
  if(verbose)
  cat('Completed!\n')
  return(list(unifracs=unifracs))
}

adonis3 <- function (formula, data = NULL, permutations = 999, method = "bray", 
		strata = NULL, contr.unordered = "contr.sum", contr.ordered = "contr.poly", 
		parallel = getOption("mc.cores"), ...)  {
	EPS <- sqrt(.Machine$double.eps)
	TOL <- 1e-07
	Terms <- terms(formula, data = data)
	lhs <- formula[[2]]
	lhs <- eval(lhs, data, parent.frame())
	formula[[2]] <- NULL
	rhs.frame <- model.frame(formula, data, drop.unused.levels = TRUE)
	op.c <- options()$contrasts
	options(contrasts = c(contr.unordered, contr.ordered))
	rhs <- model.matrix(formula, rhs.frame)
	options(contrasts = op.c)
	grps <- attr(rhs, "assign")
	qrhs <- qr(rhs)
	rhs <- rhs[, qrhs$pivot, drop = FALSE]
	rhs <- rhs[, 1:qrhs$rank, drop = FALSE]
	grps <- grps[qrhs$pivot][1:qrhs$rank]
	u.grps <- unique(grps)
	nterms <- length(u.grps) - 1
	if (nterms < 1) 
		stop("right-hand-side of formula has no usable terms")
	H.s <- lapply(2:length(u.grps), function(j) {
				Xj <- rhs[, grps %in% u.grps[1:j]]
				qrX <- qr(Xj, tol = TOL)
				Q <- qr.Q(qrX)
				tcrossprod(Q[, 1:qrX$rank])
			})
	if (inherits(lhs, "dist")) {
		if (any(lhs < -TOL)) 
			stop("dissimilarities must be non-negative")
		dmat <- as.matrix(lhs^2)
	}
	else if ((is.matrix(lhs) || is.data.frame(lhs)) && isSymmetric(unname(as.matrix(lhs)))) {
		dmat <- as.matrix(lhs^2)
		lhs <- as.dist(lhs)
	}
	else {
		dist.lhs <- as.matrix(vegdist(lhs, method = method, ...))
		dmat <- dist.lhs^2
	}
	n <- nrow(dmat)
	G <- -sweep(dmat, 1, rowMeans(dmat))/2
	SS.Exp.comb <- sapply(H.s, function(hat) sum(G * t(hat)))
	SS.Exp.each <- c(SS.Exp.comb - c(0, SS.Exp.comb[-nterms]))
	H.snterm <- H.s[[nterms]]
	tIH.snterm <- t(diag(n) - H.snterm)
	
	
	G.s <- list()
	G.s[[1]] <- G
	if (length(H.s) > 1)
		for (i in 2:(length(H.s))) 	G.s[[i]] <- (diag(n) - H.s[[i - 1]]) %*% G %*% (diag(n) - H.s[[i - 1]])
	
	
	if (length(H.s) > 1) 
		for (i in length(H.s):2) H.s[[i]] <- H.s[[i]] - H.s[[i - 1]]
	SS.Res <- sum(G * tIH.snterm)
	df.Exp <- sapply(u.grps[-1], function(i) sum(grps == i))
	df.Res <- n - qrhs$rank
	if (inherits(lhs, "dist")) {
		beta.sites <- qr.coef(qrhs, as.matrix(lhs))
		beta.spp <- NULL
	}
	else {
		beta.sites <- qr.coef(qrhs, dist.lhs)
		beta.spp <- qr.coef(qrhs, as.matrix(lhs))
	}
	colnames(beta.spp) <- colnames(lhs)
	colnames(beta.sites) <- rownames(lhs)
	F.Mod <- (SS.Exp.each/df.Exp)/(SS.Res/df.Res)
	f.test <- function(tH, G, df.Exp, df.Res, tIH.snterm) {
		(sum(G * tH)/df.Exp)/(sum(G * tIH.snterm)/df.Res)
	}
	SS.perms <- function(H, G, I) {
		c(SS.Exp.p = sum(G * t(H)), S.Res.p = sum(G * t(I - H)))
	}
	getPermuteMatrix <- getFromNamespace("getPermuteMatrix", "vegan")
	p <- getPermuteMatrix(permutations, n, strata = strata)
	permutations <- nrow(p)
	if (permutations) {
		tH.s <- lapply(H.s, t)
		if (is.null(parallel)) 
			parallel <- 1
		hasClus <- inherits(parallel, "cluster")
		isParal <- hasClus || parallel > 1
		isMulticore <- .Platform$OS.type == "unix" && !hasClus
		if (isParal && !isMulticore && !hasClus) {
			parallel <- makeCluster(parallel)
		}
		if (isParal) {
			if (isMulticore) {
				f.perms <- sapply(1:nterms, function(i) unlist(mclapply(1:permutations, 
											function(j) f.test(tH.s[[i]], G.s[[i]][p[j, ], p[j, 
																]], df.Exp[i], df.Res, tIH.snterm), mc.cores = parallel)))
			}
			else {
				f.perms <- sapply(1:nterms, function(i) parSapply(parallel, 
									1:permutations, function(j) f.test(tH.s[[i]], 
												G.s[[i]][p[j, ], p[j, ]], df.Exp[i], df.Res, tIH.snterm)))
			}
		}
		else {
			f.perms <- sapply(1:nterms, function(i) sapply(1:permutations, 
								function(j) f.test(tH.s[[i]], G.s[[i]][p[j, ], p[j, 
													]], df.Exp[i], df.Res, tIH.snterm)))
		}
		if (isParal && !isMulticore && !hasClus) 
			stopCluster(parallel)
		P <- (rowSums(t(f.perms) >= F.Mod - EPS) + 1)/(permutations + 
					1)
	}
	else {
		f.perms <- P <- rep(NA, nterms)
	}
	SumsOfSqs = c(SS.Exp.each, SS.Res, sum(SS.Exp.each) + SS.Res)
	tab <- data.frame(Df = c(df.Exp, df.Res, n - 1), SumsOfSqs = SumsOfSqs, 
			MeanSqs = c(SS.Exp.each/df.Exp, SS.Res/df.Res, NA), F.Model = c(F.Mod, 
					NA, NA), R2 = SumsOfSqs/SumsOfSqs[length(SumsOfSqs)], 
			P = c(P, NA, NA))
	rownames(tab) <- c(attr(attr(rhs.frame, "terms"), "term.labels")[u.grps], 
			"Residuals", "Total")
	colnames(tab)[ncol(tab)] <- "Pr(>F)"
	attr(tab, "heading") <- c("Terms added sequentially (first to last)\n")
	class(tab) <- c("anova", class(tab))
	out <- list(aov.tab = tab, call = match.call(), coefficients = beta.spp, 
			coef.sites = beta.sites, f.perms = f.perms, model.matrix = rhs, 
			terms = Terms)
	class(out) <- "adonis"
	out
}



PermanovaG <- function(formula, data = NULL, ...) {
	# Distance based statistical test by combining multiple distance
	# matrices based on PERMANOVA procedure by taking the maximum of
	# pseudo-F statistics
	#	
	# Args:
	#		formula: left side of the formula is a three dimensional array
	#				 of the supplied distance matrices as produced by GUniFrac,
	#                Or a list of distance matrices.
	#		dat: data.frame containing the covariates
	#		...: Parameter passing to "adonis" function
	#
	# Returns:
	# 	p.tab (data.frame): rows - Covariates (covariate of interest should appear at the last)
	#        columns - F.model, p.values for individual distance matrices and the omnibus test, 
	#   aov.tab.list:  a list of the aov.tab from individual matrices 
	
	save.seed <- get(".Random.seed", .GlobalEnv)
	lhs <- formula[[2]]
	lhs <- eval(lhs, data, parent.frame())
	rhs <- as.character(formula)[3]
	
	array.flag <- is.array(lhs)
	list.flag <- is.list(lhs)
	
	if (!array.flag & !list.flag) {
		stop('The left side of the formula should be either a list or an array of distance matrices!\n')
	}
	if (array.flag) {
		if (length(dim(lhs)) != 3) {
			stop('The array should be three-dimensional!\n')
		} else {
			len <- dim(lhs)[3]
		}
	}
	
	if (list.flag) {
		len <- length(lhs)
	}
	
	p.perms <- list()
	p.obs <- list()
	aov.tab.list <- list()
	for (i_ in 1:len) {
		assign(".Random.seed", save.seed, .GlobalEnv)
		if (array.flag) {
			Y <- as.dist(lhs[, , i_])
		}
		if (list.flag) {
			Y <- as.dist(lhs[[i_]])
		}
		
		obj <- adonis(as.formula(paste("Y", "~", rhs)), data, ...)
		perm.mat <- obj$f.perms
		p.perms[[i_]] <- 1 - (apply(perm.mat, 2, rank) - 1) / nrow(perm.mat)
		p.obs[[i_]] <- obj$aov.tab[1:ncol(perm.mat), "Pr(>F)"]
		aov.tab.list[[i_]] <- obj$aov.tab
	}
	
	omni.pv <- NULL
	indiv.pv <- NULL
	for (j_ in 1:ncol(perm.mat)) {
		p.perms.j <- sapply(p.perms, function (x) x[, j_])
		p.obj.j <- sapply(p.obs, function (x) x[j_])
		omni.pv <- c(omni.pv, mean(c(rowMins(p.perms.j ) <= min(p.obj.j), 1)))
		indiv.pv <- rbind(indiv.pv, p.obj.j)
	}
	colnames(indiv.pv) <- paste0('D', 1:ncol(indiv.pv), '.p.value')
	rownames(indiv.pv) <- 1:nrow(indiv.pv)
	
	p.tab <- data.frame(indiv.pv, omni.p.value = omni.pv)
	rownames(p.tab) <- rownames(obj$aov.tab)[1:ncol(perm.mat)]
	
	
	list(p.tab = p.tab, aov.tab.list = aov.tab.list)
}



PermanovaG2 <- function(formula, data = NULL, ...) {
	# Distance based statistical test by combining multiple distance
	# matrices based on PERMANOVA procedure by taking the maximum of
	# pseudo-F statistics
	#	
	# Args:
	#		formula: left side of the formula is a three dimensional array
	#				 of the supplied distance matrices as produced by GUniFrac,
	#                Or a list of distance matrices.
	#		dat: data.frame containing the covariates
	#		...: Parameter passing to "adonis" function
	#
	# Returns:
	# 	p.tab (data.frame): rows - Covariates (covariate of interest should appear at the last)
	#        columns - F.model, p.values for individual distance matrices and the omnibus test, 
	#   aov.tab.list:  a list of the aov.tab from individual matrices 
	
	save.seed <- get(".Random.seed", .GlobalEnv)
	lhs <- formula[[2]]
	lhs <- eval(lhs, data, parent.frame())
	rhs <- as.character(formula)[3]
	
	array.flag <- is.array(lhs)
	list.flag <- is.list(lhs)
	
	if (!array.flag & !list.flag) {
		stop('The left side of the formula should be either a list or an array of distance matrices!\n')
	}
	if (array.flag) {
		if (length(dim(lhs)) != 3) {
			stop('The array should be three-dimensional!\n')
		} else {
			len <- dim(lhs)[3]
		}
	}
	
	if (list.flag) {
		len <- length(lhs)
	}
	
	p.perms <- list()
	p.obs <- list()
	aov.tab.list <- list()
	for (i_ in 1:len) {
		assign(".Random.seed", save.seed, .GlobalEnv)
		if (array.flag) {
			Y <- as.dist(lhs[, , i_])
		}
		if (list.flag) {
			Y <- as.dist(lhs[[i_]])
		}
		
		obj <- adonis3(as.formula(paste("Y", "~", rhs)), data, ...)
		perm.mat <- obj$f.perms
		p.perms[[i_]] <- 1 - (apply(perm.mat, 2, rank) - 1) / nrow(perm.mat)
		p.obs[[i_]] <- obj$aov.tab[1:ncol(perm.mat), "Pr(>F)"]
		aov.tab.list[[i_]] <- obj$aov.tab
	}
	
	omni.pv <- NULL
	indiv.pv <- NULL
	for (j_ in 1:ncol(perm.mat)) {
		p.perms.j <- sapply(p.perms, function (x) x[, j_])
		p.obj.j <- sapply(p.obs, function (x) x[j_])
		omni.pv <- c(omni.pv, mean(c(rowMins(p.perms.j ) <= min(p.obj.j), 1)))
		indiv.pv <- rbind(indiv.pv, p.obj.j)
	}
	colnames(indiv.pv) <- paste0('D', 1:ncol(indiv.pv), '.p.value')
	rownames(indiv.pv) <- 1:nrow(indiv.pv)
	
	p.tab <- data.frame(indiv.pv, omni.p.value = omni.pv)
	rownames(p.tab) <- rownames(obj$aov.tab)[1:ncol(perm.mat)]
	
	
	list(p.tab = p.tab, aov.tab.list = aov.tab.list)
}


Rarefy <- function (otu.tab, depth = min(rowSums(otu.tab))){
	# Rarefaction function: downsample to equal depth
	#	
	# Args:
	#		otu.tab: OTU count table, row - n sample, column - q OTU
	#		depth: required sequencing depth 
	#
	# Returns:
	# 	otu.tab.rff: Rarefied OTU table
	#		discard: labels of discarded samples
	#
	otu.tab <- as.matrix(otu.tab)
	ind <- (rowSums(otu.tab) < depth)
	sam.discard <- rownames(otu.tab)[ind]
	otu.tab <- otu.tab[!ind, ]
	
	rarefy <- function(x, depth){
		y <- sample(rep(1:length(x), x), depth)
		y.tab <- table(y)
		z <- numeric(length(x))
		z[as.numeric(names(y.tab))] <- y.tab
		z
	}
	otu.tab.rff <- t(apply(otu.tab, 1, rarefy, depth))
	rownames(otu.tab.rff) <- rownames(otu.tab)
	colnames(otu.tab.rff) <- colnames(otu.tab)
	return(list(otu.tab.rff=otu.tab.rff, discard=sam.discard))
}

calculateK <- function (G, H, df2) {
	n <- nrow(G)
	dG <- diag(G)
	
	mu1 <- sum(dG) / (n - df2)
	mu2 <- (sum(G^2) - sum(dG^2)) / ((n - df2)^2 + sum(H^4) - 2 * sum(diag(H^2)))
	
	k <- mu1^2 / mu2
	list(K = k, mu1 = mu1, mu2 = mu2)
}



dmanova <- function (formula, data = NULL, positify = FALSE,
		contr.unordered = "contr.sum", contr.ordered = "contr.poly", 
		returnG = FALSE) {
	Terms <- terms(formula, data = data)
	lhs <- formula[[2]]
	lhs <- eval(lhs, data, parent.frame())
	
	formula[[2]] <- NULL
	rhs.frame <- model.frame(formula, data, drop.unused.levels = TRUE)
	op.c <- options()$contrasts
	options(contrasts = c(contr.unordered, contr.ordered))
	rhs <- model.matrix(formula, rhs.frame)
	options(contrasts = op.c)
	grps <- attr(rhs, "assign")
	
	# The first group includes the intercept, nterms count the intercept
	u.grps <- unique(grps)
	nterms <- length(u.grps)
	
	Z <- rhs[, grps %in% u.grps[1:(nterms-1)], drop=F]
	XZ <- rhs[, grps %in% u.grps[1:(nterms)], drop=F]
	
	n <- nrow(XZ)
	if (inherits(lhs, 'dist')) {
		D <- as.matrix(lhs)
	} else {
		D <- lhs
	}
	
	D <- -D^2 / 2
	
	G <- mean(D) + D - rowMeans(D) - matrix(rep(1, n), ncol=1) %*% colMeans(D)
	
	if (positify) {
		G <- as.matrix(nearPD(G)$mat)
	}
	
	XZi <- solve(t(XZ) %*% XZ)
	Zi <- solve(t(Z) %*% (Z))
	HZ <- Z %*% Zi %*% t(Z)
	HXZ <- XZ %*% XZi %*% t(XZ)
	HX <- HXZ - HZ
	HIXZ <- diag(n) - HXZ
	HIX <- diag(n) - HX
	
	
	df1 <- ncol(XZ) - ncol(Z)
	df2 <- n - ncol(XZ)
	
	MSS <- sum(G * HX)
	RSS <- sum(G * HIXZ)
	TSS <- sum(diag(G))
	
	
	f.stat <- (MSS / df1) / (RSS / df2)
	
	GXZ <- G %*% XZ
	XZXZi <- XZ %*% XZi
	GXZtXZXZi <- GXZ %*% t(XZXZi)
	
	G.tilde <- G + XZXZi %*% (t(GXZ) %*% XZ) %*% t(XZXZi) - GXZtXZXZi - t(GXZtXZXZi)
	
		
	obj <- calculateK(G.tilde, HIXZ, n - df2)
	K <- obj$K
	
	
	p.value <- pchisq(f.stat * K * df1, df = K * df1, lower.tail = F)
		


	SumsOfSqs <- c(MSS, RSS, TSS)
	
	tab <- data.frame(Df = c(df1, df2, n - 1), SumsOfSqs = SumsOfSqs, 
			MeanSqs = c(MSS/df1, RSS/df2, NA), F.Model = c(f.stat, 
					NA, NA), R2 = c(MSS/TSS, NA, NA), 
			"Pr(>F)" = c(p.value, NA, NA))
	
	rownames(tab) <- c("Model(Adjusted)", "Residuals", "Total")
	colnames(tab)[ncol(tab)] <- c("Pr(>F)")
	class(tab) <- c("anova", class(tab))
	attr(tab, "heading") <- c("F stat and P value of the last term is adjusted by preceding terms!\n")
	
	if (returnG == TRUE) {
		out <- list(aov.tab = tab,  df = K, G = G,  call = match.call())
	} else {
		out <- list(aov.tab = tab, df = K, call = match.call())
	}
	
	out
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

PairwiseD <- function(cum, b, a) {
    .Call('_GUniFrac_PairwiseD', PACKAGE = 'GUniFrac', cum, b, a)
}

gmpr <- function(x, min_ct, intersect_no) {
    .Call('_GUniFrac_gmpr', PACKAGE = 'GUniFrac', x, min_ct, intersect_no)
}

---
title: "Performing differential abundance analysis using ZicoSeq"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Performing differential abundance analysis using ZicoSeq}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

# 1. Introduction
`ZicoSeq` is a linear model and permutation-based method for dfferential
abundance analysis of zero-inflated compositional sequencing data such as
microbiome sequencing data. It has the following components:
 
 - Winsorization to decrease the influence of outliers; 
 - Posterior sampling based on a beta mixture prior to address sampling variability and zero inflation; 
 - Reference-based multiple-stage normalization to address compositional effects; 
 - An omnibus test to address diverse feature-covariate relationships; 
 - Permutation-based false discovery rate control / family-wise error rate
   control for multiple testing correction, which takes into account the
   correlation structure in the feature data.

Currently `ZicoSeq` supports:

- **count data** or **proportion data**. For both count and proportion data, a
  reference-based ratio approach is used to account for compositional effects.
  When a count matrix is provided, it provides an option to draw posterior
  samples of the underlying proportions to account for the sampling variability
  during the sequencing process. The test results are aggregated over these
  posterior samples.

- **other data types**. As a general methodology, `ZicoSeq` can be used
  to differential analysis of other high-dimensional datasets such as
  transcriptomics, epigenomics, metabolomics, and proteomics data. 
  
# 2. Installation

Install the package.
```{r}
# install.packages("GUniFrac")
```

Load the package.
```{r, message=FALSE}
library(GUniFrac)
```

# 3. Running `ZicoSeq`
## 3.1 Loading the example data
This example data set contains the OTU count data from the throat microbiome of
the left body side, which is  available in `GUniFrac` package. It contains 60
subjects consisting of 32 nonsmokers and 28 smokers [(Charlson, Emily S., et
al.,
2010).](https://journals.plos.org/plosone/article?id=10.1371/journal.pone.0015216).
In this example, we will identify smoking-associated OTUs while adjusting the
sex since sex is a potential confounder (we see more males in smokers).
```{r, results = FALSE}
data(throat.otu.tab)
data(throat.meta)
comm <- t(throat.otu.tab)
meta.dat <- throat.meta
meta.dat

```


## 3.2 Running `ZicoSeq` function
```{r, results = FALSE}
ZicoSeq.obj <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm, 
                     grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "count",
                    # Filter to remove rare taxa
                    prev.filter = 0.2, mean.abund.filter = 0,  
                    max.abund.filter = 0.002, min.prop = 0, 
                    # Winsorization to replace outliers
                    is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
                    # Posterior sampling 
                    is.post.sample = TRUE, post.sample.no = 25, 
                    # Use the square-root transformation
                    link.func = list(function (x) x^0.5), stats.combine.func = max,
                    # Permutation-based multiple testing correction
                    perm.no = 99,  strata = NULL, 
                    # Reference-based multiple stage normalization
                    ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
                    # Family-wise error rate control
                    is.fwer = TRUE, verbose = TRUE, return.feature.dat = TRUE)
```

## 3.3 Some notes on using `ZicoSeq`
### 3.3.1 Winsorization

Microbiome data may contain outliers  with extremely large counts.  These
outliers could reduce the statistical power, or more seriously increase the type
I error rate. In `ZicoSeq`, we use winsorization to replace those extremely
large counts with a certain quantile (e.g. 97%). We recommend always setting
`is.winsor = TRUE` for real data analysis. The specific quantile used depends on
the expected proportion of outliers.

### 3.3.2 Posterior sampling

Posterior sampling can be enabled by setting `is.post.sample = TRUE`. It can
only be used when the data type is "count".  For microbiome data, the majority
of the taxa are rare, resulting in a large number of zeros in the data given a
limited sequencing depth.  As the probability of zero strongly depends on the
sequencing depth, zeros from different samples are not comparable.  In
`ZicoSeq`, we provide a method to infer/impute the underlying true proportions
using an empirical Bayes approach by pooling information across samples. The
imputed proportions depend on both the sequencing depth and the estimated
(prior) distribution of the proportions across samples. To better model the
prior distribution, we used beta mixtures instead a single-component beta
distribution.  **Posterior sampling effectively reduces the type I error when
the sequencing depth is associated with the covariate of interest (e.g. cases
and controls differ in sequencing depth) and is more powerful than
rarefaction.**  When the sequencing depth is not a confounding factor,
`is.post.sample` slightly improves the power and could be disabled for analyzing
large datasets.

### 3.3.3 Omnibus testing

The relationship between the taxa abundance and the covariate of interest
may vary across taxa, a single transformation function may not be powerful
enough to capture diverse relationships. `ZicoSeq` allows specifying multiple
possible transformation functions (_`link.func`_) and performs omnibus testing.
Although the log transformation is commonly used for microbiome due to its
interpretability, we found that power transformations may be more powerful. The
default is the square-root transformation. 

### 3.3.4 Reference selection
`ZicoSeq` uses a reference-based approach to address compositional effects. The
default uses 30% of the least variable/significant taxa as the reference.  It
starts with selecting 50% of the least variable taxa as the reference and
differential abundance testing is then run multiple times excluding 20% taxa
with the smallest p-values in each iteration. This procedure is to make sure the
remaining 30% are more likely to be non-differential.

### 3.3.5. Error control
`ZicoSeq` uses a permutation-based approach to control false discovery rate
(FDR). The permutation-based FDR control preserves the correlation structure in
the abundance data and is more robust and powerful than the traditional BH-based
FDR control based on raw p values.  `ZicoSeq`  also allows performing
permutation-based family-wise error rate control. Users can determine the
appropriate error control method to suit their needs.


## 3.4 `ZicoSeq` output
The output of `ZicoSeq` consists of mainly: 

### 3.4.1 p-values
  
  - `p.raw`:  raw p-values based on permutations, it will not be accurate if
    `perm.no` is small.
  - `p.adj.fdr`: permutation-based FDR-adjusted p-values.
  - `p.adj.fwer`: permutation-based FWER-adjusted (West-Young), only if
    `is.fwer = T` in `ZicoSeq`. `perm.no`  is recommened to set at least 999 for
    accurate FWER-adjusted p-values.


### 3.4.2 R^2 (percentage of variance explained)
A matrix of R^2 values (number of features by number of transformation
functions) will be provided. R^2 is an effect size measure and can be used to
assess the association strength between the taxa abundance and the covariate of
interest while adjusting for other covariates.  When the omnibus testing is in
action, i.e.,  multiple transformations are used, there will be multiple R^2 s,
each corresponding to a specific transformation. 



## 3.5 `ZicoSeq` output visualization
`ZicoSeq.plot` function produces a volcano plot with the y-axis being the
log10 (adjusted) p-value and the x-axis being the signed  R^2 with the sign
indicating the association direction determined based on  the sign of the
regression coefficient (for mutli-categorical variables, sign is not
applicable). When multiple transformation functions are used, the largest
R^2 is used. The names of differential taxa passing a specific significance
cutoff will be printed on the figure. When data types are counts and
proportions, the mean abundance and prevalence will be visualized; when the data
type is `other`, mean and standard deviation of the features will be visualized.
Users need to set `return.feature.dat = T` when using the plot function.

```{r, fig.retina = 4, fig.width= 8, fig.height=6, results=FALSE, message=FALSE, warning=FALSE}

ZicoSeq.plot(ZicoSeq.obj, pvalue.type = 'p.adj.fdr', cutoff = 0.1, text.size = 10,
             out.dir = NULL, width = 10, height = 6)
```


## 3.6 More examples 

### 3.6.1 Proportion data
For some bioinformatics pipelines, the output could be proportion data.
`ZicoSeq`  can be applied to proportion data by specifying `feature.dat.type =
"proportion"` . Posterior sampling will not be applied when analyzing the
proportions and other parameter settings are similar to the count case.
```{r, results = FALSE,  fig.width=8, fig.height=6}
comm.p <- t(t(comm) / colSums(comm))
ZicoSeq.obj.p <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm.p, 
                    grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "proportion",
                    # Filter to remove rare taxa
                    prev.filter = 0.2, mean.abund.filter = 0,  max.abund.filter = 0.002, min.prop = 0, 
                    # Winsorization to replace outliers
                    is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
                    # Posterior sampling will be automatically disabled
                    is.post.sample = FALSE, post.sample.no = 25, 
                    # Use the square-root transformation
                    link.func = list(function (x) x^0.5, function (x) x^0.25), stats.combine.func = max,
                    # Permutation-based multiple testing correction
                    perm.no = 99,  strata = NULL, 
                    # Reference-based multiple stage normalization
                    ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
                    # Family-wise error rate control
                    is.fwer = TRUE, verbose = TRUE, return.feature.dat = T)
suppressWarnings(ZicoSeq.plot(ZicoSeq.obj = ZicoSeq.obj.p, pvalue.type = 'p.adj.fdr', 
             cutoff = 0.1, text.size = 10, out.dir = NULL, width = 10, height = 6))                    

```

### 3.6.2 General data types 
`ZicoSeq`  as a  general linear model-based permutation test can be applied to
association analyses of other high-dimensional datasets such as transcriptome-,
methylome-, metabolome- and proteome-wide association testing, where the omics
features are treated as the outcomes. In this case, posterior sampling will be
automatically disabled. The user should be responsible for properly normalizing,
transforming, and filtering the data before applying  `ZicoSeq`. The user should
also decide whether to winsorize the top, bottom or both ends of the
distribution.  The following code is just for demonstration purposes and does
not mean to be applied for differential abundance analysis. 

```{r, fig.retina = 4, fig.width= 8, fig.height=6, results=FALSE, message=FALSE, warning=FALSE}
comm.o <- comm[rowMeans(comm != 0) >= 0.2, ] + 1
comm.o <- log(t(t(comm.o) / colSums(comm.o)))
ZicoSeq.obj.o <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm.o, 
		grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "other",
		# Filter will not be applied
		prev.filter = 0, mean.abund.filter = 0,  max.abund.filter = 0, min.prop = 0, 
		# Winsorization the top end 
		is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
		# Posterior sampling will be automatically disabled
		is.post.sample = FALSE, post.sample.no = 25, 
		# Identity function is used
		link.func = list(function (x) x), stats.combine.func = max,
		# Permutation-based multiple testing correction
		perm.no = 99,  strata = NULL, 
		# Reference-based multiple-stage normalization will not be performed
		ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
		# Family-wise error rate control
		is.fwer = TRUE, verbose = TRUE, return.feature.dat = T)
    ZicoSeq.plot(ZicoSeq.obj = ZicoSeq.obj.o,  pvalue.type = 'p.adj.fdr', 
             cutoff = 0.1, text.size = 10, out.dir = NULL, width = 10, height = 6)    
```


### 3.6.3 Random effect model - within-subject comparisons
`ZicoSeq`  can be applied to perform within-subject comparisons (e.g. before
treatment vs. after treatment) by using the `strata` parameter.  For example,
if the subject variable is "subject_id" in the meta data file, the user can set
`strata = 'subject_id'` .


# 4. References

  - Lu Yang & Jun Chen. 2022. A comprehensive evaluation of differential abundance analysis methods: current status and potential solutions. Microbiome. 10, Article number: 130.

  - Charlson ES, Chen J, Custers-Allen R, Bittinger K, Li H, Sinha R, Hwang J, Bushman FD, Collman RG. Disordered microbial communities in the upper respiratory tract of cigarette smokers. PLoS One. 2010 Dec 20;5(12):e15216. 


# 5. Session info
```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.R
\name{SimulateMSeq}
\alias{SimulateMSeq}
\title{A Semiparametric Model-based Microbiome Sequencing Data Simulator for Cross-sectional and Case-control Studies}
\description{
The function generates synthetic microbiome sequencing data for studying the performance of differential abundance analysis methods.  
It uses a user-supplied (large) reference OTU table as a template to generate a synthetic OTU table of specified size. A subset of OTUs  are affected
by a simulated covariate of interest, either binary or continuous.  Confounder effects can also be simulated.  The function allows simulating
different signal structures, i.e., the percentage of differential OTUs, their effect sizes, their direction of change,  and whether these OTUs are
relatively abundant or rare.
}

\usage{
SimulateMSeq(
  ref.otu.tab,
  nSam = 100,
  nOTU = 500,
  diff.otu.pct = 0.1,
  diff.otu.direct = c("balanced", "unbalanced"),
  diff.otu.mode = c("abundant", "rare", "mix"),
  covariate.type = c("binary", "continuous"),
  grp.ratio = 1,
  covariate.eff.mean = 1,
  covariate.eff.sd = 0,
  confounder.type = c("none", "binary", "continuous", "both"),
  conf.cov.cor = 0.6,
  conf.diff.otu.pct = 0,
  conf.nondiff.otu.pct = 0.1,
  confounder.eff.mean = 0,
  confounder.eff.sd = 0,
  error.sd = 0,
  depth.mu = 10000,
  depth.theta = 5,
  depth.conf.factor = 0
)
}
\arguments{
\item{ref.otu.tab}{a matrix, the reference OTU count table (row - OTUs, column - samples), serving as the template for synthetic sample generation.}


\item{nSam}{the number of samples to be simulated.}

\item{nOTU}{the number of OTUs to be simulated. }

\item{diff.otu.pct}{a numeric value between 0 and 1, the percentage of differential OTUs to be simulated. If 0, global null setting is simulated. The default is 0.1.}

\item{diff.otu.direct}{a character string of "balanced" or "unbalanced".  "balanced" - the direction of change for these differential OTUs is random, "unbalanced" - direction of change is the same. The default is "balanced".}

\item{diff.otu.mode}{a character string of  "rare", "mix" or "abundant".  "abundant" - differential OTU come from the top quartile of the abundance distribution, 
"rare" - differential OTU come from the bottom quartile of the abundance distribution, and "mix" - random set. The default is "abundant". }

\item{covariate.type}{a character string of "binary" or "continuous", indicating the type of the covariate to be simulated. The default is "binary" (e.g., case v.s. control).}

\item{grp.ratio}{a numeric value between 0 and 1. Group size ratio. The default is 1, i.e., equal group size. Only relevant when \code{covariate.type} is "binary". }

\item{covariate.eff.mean}{a numeric value, the mean log fold change (effect size) in response to one unit change of the covariate.  The default is 1.}

\item{covariate.eff.sd}{a positive numeric value, the standard deviation of the log fold change.  The default is 0, i.e., the log fold change is the same across differential OTUs.}

\item{confounder.type}{a character string of "none", "binary", "continuous" or "both". The default is "none", no confounder will be simulated.
 If "both", both a binary and continuous confounder will be simulated. The default is "none".}

\item{conf.cov.cor}{a numeric value between 0 and 1.  The correlation between the covariate of interest and the confounder. The default is 0.6.}

\item{conf.diff.otu.pct}{a numeric value between 0 and 1. The percentage of OTUs affected by the confounder and the covariate of interest. The default is 0.}

\item{conf.nondiff.otu.pct}{a numeric value between 0 and 1. The percentage of OTUs affected by the confounder but not the covariate of interest. The default is 0.1.}

\item{confounder.eff.mean}{a numeric value, the mean log fold change (effect size) in response to one unit change of the confounder.  The default is 1.}

\item{confounder.eff.sd}{a positive numeric value, the standard deviation of the log fold change for the confounder.  The default is 0, i.e., the log fold change is the same across OTUs affected by the confounder.}

\item{error.sd}{the sd of the log fold change unexplained by the covariate and the confounder (i.e., the error term under the log linear model).  The default is 0. }

\item{depth.mu}{the mean sequencing depth to be simulated. The default is 10,000.}

\item{depth.theta}{the theta value  of the negative binomial distribution controlling the variance (\code{mu + mu^2/theta}). The default is 5.}

\item{depth.conf.factor}{a numeric value controlling the dependence of the sequencing depth on the covariate of interest (\code{depth.mu} * \code{exp(scale(X)} * \code{depth.conf.factor})). 
The default is 0, i.e., the depth is not associated with the covariate of interest. This parameter can be used to simulate depth confounding.}
}
\value{
Return a list with the elements:
\item{otu.tab.sim}{simulated OTU table}
\item{covariate}{simulated covariate of interest}
\item{confounder}{simulated confounder(s)}
\item{diff.otu.ind}{indices of the differential OTUs, i.e., affected by the covariate of interest}
\item{otu.names}{the names of the simulated OTUs}
\item{conf.otu.ind}{indices of OTUs affected by the confounder(s)}
}

\details{

This function implements a semiparametric approach for realistic independent microbiome sequencing data generation. The method draws random samples from a large reference dataset (non-parametric part) and uses
 these reference samples as templates to generate new samples (parametric part). Specifically, for each drawn reference sample, it infers the underlying composition based on a Bayesian model and 
 then adds covariate/confounder effects to the composition vector, based on which a new sequencing sample is generated. The method circumvents the difficulty in modeling the inter-subject variation
 of the microbiome composition.   

}
\references{
Yang, L. & Chen, J. 2022. A comprehensive evaluation of differential abundance analysis methods: current status and potential solutions.
Microbiome. In Press.
}


\author{Jun Chen and Lu Yang}

\examples{

# Use throat microbiome for illustration
data(throat.otu.tab)
comm <- t(throat.otu.tab)
comm <- comm[rowMeans(comm != 0) > 0.2, ]

# Simulate binary covariate, 10\% signal density, abundant differential OTUs, unbalanced change 
# This setting simulates strong compositional effects
sim.obj <- SimulateMSeq(
		ref.otu.tab = comm, nSam = 50, nOTU = 50,
		# True signal setting
		diff.otu.pct = 0.1, diff.otu.direct = c("unbalanced"), 
		diff.otu.mode = c("abundant"),
		covariate.type = c("binary"), grp.ratio = 1,
		covariate.eff.mean = 1.0, covariate.eff.sd = 0,
		# Confounder signal setting
		confounder.type = c("both"), conf.cov.cor = 0.6,
		conf.diff.otu.pct = 0.1, conf.nondiff.otu.pct = 0.1,
		confounder.eff.mean = 1.0, confounder.eff.sd = 0,
		# Depth setting
		depth.mu = 10000, depth.theta = 5, depth.conf.factor = 0
)

meta.dat <- data.frame(X = sim.obj$covariate, Z1 = sim.obj$confounder[, 1],
                       Z2 = sim.obj$confounder[, 2])
otu.tab.sim <- sim.obj$otu.tab.sim

# Run ZicoSeq for differential abundance analysis
zico.obj <- ZicoSeq(meta.dat = meta.dat, feature.dat = otu.tab.sim, 
		grp.name = 'X', adj.name = c('Z1', 'Z2'), feature.dat.type = "count",
		# Filter to remove rare taxa
		prev.filter = 0.2, mean.abund.filter = 0,  max.abund.filter = 0.002, min.prop = 0, 
		# Winsorization to replace outliers
		is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
		# Posterior sampling to impute zeros
		is.post.sample = TRUE, post.sample.no = 25, 
		# Multiple link functions to capture diverse taxon-covariate relation
		link.func = list(function (x) x^0.25, function (x) x^0.5, function (x) x^0.75), 
		stats.combine.func = max,
		# Permutation-based multiple testing correction
		perm.no = 99,  strata = NULL, 
		# Reference-based multiple stage normalization
		ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
		# Family-wise error rate control
		is.fwer = FALSE,
		verbose = TRUE, return.feature.dat = FALSE)

# Detected differential OTUs
which(zico.obj$p.adj.fdr <= 0.05)

# True differential OTUs
sim.obj$otu.names[sim.obj$diff.otu.ind]
}

\keyword{composition}
\keyword{microbiome}
\keyword{simulation}

\name{adonis3}
\alias{adonis3}


\title{Permutational Multivariate Analysis of Variance Using Distance Matrices (Freedman-Lane permutation)}

\description{Analysis of variance using distance matrices --- for
  partitioning distance matrices among sources of variation and fitting
  linear models (e.g., factors, polynomial regression) to distance 
  matrices; uses a permutation test (Freedman-Lane permutation) with pseudo-\eqn{F} ratios.}

\usage{
adonis3(formula, data, permutations = 999, method = "bray",
    strata = NULL, contr.unordered = "contr.sum",
    contr.ordered = "contr.poly", parallel = getOption("mc.cores"), ...)
}

\arguments{

  \item{formula}{model formula. The LHS must be either a community
    data matrix or a dissimilarity matrix, e.g., from
    \code{\link[vegan]{vegdist}} or \code{\link{dist}}.  If the LHS is a data
    matrix, function \code{\link[vegan]{vegdist}} will be used to find the
    dissimilarities. The RHS defines the independent variables. These
    can be continuous variables or factors, they can be transformed
    within the formula, and they can have interactions as in a typical
    \code{\link{formula}}. }

  \item{data}{ the data frame for the independent variables.}
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}
  \item{method}{ the name of any method used in \code{\link[vegan]{vegdist}} to
    calculate pairwise distances if the left hand side of the
    \code{formula} was a data frame or a matrix. }
  \item{strata}{ groups (strata) within which to constrain permutations.  }
  \item{contr.unordered, contr.ordered}{contrasts used for the design
    matrix (default in R is dummy or treatment contrasts for unordered
    factors). }
  \item{parallel}{number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing. The parallel processing is done with \pkg{parallel}
    package.}
  \item{\dots}{Other arguments passed to \code{vegdist}.}
}

\details{

\code{adonis3} is the re-implementation of the \code{adonis} function in
the \code{vegan} package based on the Freedman-Lane permutation scheme
(Freedman & Lane (1983), Hu & Satten (2020)).  The original
implementation in the \code{vegan} package is directly based on the algorithm of Anderson (2001) and
performs a sequential test of terms. Statistical significance is assessed
based on permuting the distance matrix. We found that such permutation
will lead to power loss in testing the effect of a covariate of interest while adjusting
for other covariates (confounders). The power loss is more evident  when the confounders' effects
are strong, the correlation between the covariate of interest and the confounders is high, and
the sample size is small. When the sample size is large than 100, the difference is usually small.
The new implementation is revised on the \code{adonis} function with the same interface.
}

\value{


  Function \code{adonis3} returns an object of class \code{"adonis"} with
  following components:

  \item{aov.tab}{typical AOV table showing sources of variation,
    degrees of freedom, sequential sums of squares, mean squares,
    \eqn{F} statistics, partial \eqn{R^2}{R-squared} and \eqn{P}
    values, based on \eqn{N} permutations. }
  \item{coefficients}{ matrix of coefficients of the linear model, with
    rows representing sources of variation and columns representing
    species; each column represents a fit of a species abundance to the
    linear model. These are what you get when you fit one species to
    your predictors. These are NOT available if you supply the distance
    matrix in the formula, rather than the site x species matrix} 
  \item{coef.sites}{ matrix of coefficients of the linear model, with
    rows representing sources of variation and columns representing
    sites; each column represents a fit of a sites distances (from all
    other sites) to the  linear model. These are what you get when you
    fit distances of one site to
    your predictors. }   
  \item{f.perms}{ an \eqn{N} by \eqn{m} matrix of the null \eqn{F}
    statistics for each source of variation based on \eqn{N}
    permutations of the data. The permutations can be inspected with
    \code{\link[vegan]{permustats}} and its support functions.}
  \item{model.matrix}{the \code{\link{model.matrix}} for the right hand
    side of the formula.}
  \item{terms}{the \code{\link{terms}} component of the model.}
}



\references{
Anderson, M.J. 2001. A new method for non-parametric multivariate
analysis of variance. \emph{Austral Ecology}, \strong{26}: 32--46.

Freedman D. & Lane D. 1983. A nonstochastic interpretation of reported significance
levels. \emph{Journal of Business and Economic Statistics}, \strong{1}292--298.

Hu, Y. J. & Satten, G. A. 2020. Testing hypotheses about the microbiome using the 
linear decomposition model (LDM). \emph{JBioinformatics}, \strong{ 36(14)} : 4106-4115.


}

\author{Martin Henry H. Stevens (\code{adonis}) and Jun Chen
  (\code{adonis3}). }


\examples{
\dontrun{
data(throat.otu.tab)
data(throat.tree)
data(throat.meta)

groups <- throat.meta$SmokingStatus

# Rarefaction
otu.tab.rff <- Rarefy(throat.otu.tab)$otu.tab.rff

# Calculate the UniFrac distance
unifracs <- GUniFrac(otu.tab.rff, throat.tree, alpha=c(0, 0.5, 1))$unifracs

# Test the smoking effect based on unweighted UniFrac distance, adjusting sex
adonis3(as.dist(unifracs[, , 'd_UW']) ~ Sex + SmokingStatus, data = throat.meta)
}

}

\keyword{distance}
\keyword{multivariate}

\name{throat.otu.tab}
\alias{throat.otu.tab}
\docType{data}
\title{
%%   ~~ data name/kind ... ~~
Throat Microbiome OTU Count Table
}
\description{
%%  ~~ A concise (1-5 lines) description of the dataset. ~~
It is part of a microbiome data set (16S V12-targeted 454 pyrosequencing) for studying the effect of
 smoking on  the upper respiratory tract microbiome. The original data 
set contains samples from both throat and  nose microbiomes, and 
from both body sides. This data set comes from the throat microbiome 
of left body side. It contains 60 subjects consisting of 32 nonsmokers
 and 28 smokers. 
}
\usage{data(throat.otu.tab)}

\details{
%%  ~~ If necessary, more details than the __description__ above ~~
The OTU table is produced by the QIIME software. Singleton OTUs have
been discarded.
}
\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
Charlson ES, Chen J, Custers-Allen R, Bittinger K, Li H, et al. (2010) 
Disordered Microbial Communities in the Upper Respiratory Tract of 
Cigarette Smokers. PLoS ONE 5(12): e15216. 
}

\examples{
data(throat.otu.tab)
}
\keyword{datasets}

\name{PermanovaG2}
\alias{PermanovaG2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
%%  ~~function to do ... ~~
Permutational Multivariate Analysis of Variance Using Multiple Distance Matrices(Freedman-Lane Permutation)

}
\description{
%%  ~~ A concise (1-5 lines) description of what the function does. ~~
In practice, we do not know a priori which type of change happens in 
the microbiome. Each distance measure is most powerful in detecting
only a certain scenario. When multiple distance matrices are available,
separate tests using each distance matrix will lead to loss of power 
due to multiple testing correction. Combing the distance matrices
in a single test will improve power. PermanovaG combines multiple distance
matrices  by taking the minimum of the P values for individual distance
matrices. Significance is assessed by permutation.
}
\usage{
PermanovaG2(formula, data = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{formula}{
%%     ~~Describe \code{formula} here~~
a formula, left side of the formula (\code{Y ~ X}) is a three dimensional ARRAY
containing the supplied distance matrices as produced by \code{\link{GUniFrac}}
 function. Or it could be a list of distance matrices.
}
  \item{data}{
%%     ~~Describe \code{dat} here~~
a data frame containing the covariates
}
  \item{\dots}{
%%     ~~Describe \code{\dots} here~~
parameters passing to \code{adonis} function
}
}

\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
Return a list containing:
\item{p.tab}{a data frame, columns: p-values for individual distance matrices and the omnibus test, 
rows: covariates. (Note: they are sequential p-values, put the variable of interest in the end)}
\item{aov.tab.list}{a list of \code{adonis} AOV tables for individual distance matrices}

}

\references{
%% ~put references to the literature/web site here ~
Chen, J., Bittinger, K., Charlson, E.S., Hoffmann, C., Lewis, J., Wu, G.D., Collman, R.G., Bushman, F.D. and Li, H. (2012). Associating microbiome composition with 
environmental covariates using generalized UniFrac distances. 28(16): 2106–2113.
}
\author{
%%  ~~who you are~~
Jun Chen <chen.jun2@mayo.edu>
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{Rarefy}}, \code{\link{GUniFrac}}, \code{\link{adonis3}}
}
\examples{
\dontrun{
data(throat.otu.tab)
data(throat.tree)
data(throat.meta)

groups <- throat.meta$SmokingStatus

# Rarefaction
otu.tab.rff <- Rarefy(throat.otu.tab)$otu.tab.rff

# Calculate the UniFracs
unifracs <- GUniFrac(otu.tab.rff, throat.tree, alpha=c(0, 0.5, 1))$unifracs

#	Combine unweighted and weighted UniFrac for testing
PermanovaG2(unifracs[, , c("d_1", "d_UW")] ~ groups)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

\keyword{ distance }
\keyword{ multivariate }
\keyword{ nonparametric }
\keyword{ regression }% __ONLY ONE__ keyword per line

\name{GMPR}
\alias{GMPR}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Geometric Mean of Pairwise Ratios (GMPR) Normalization for Zero-inflated Count Data
}
\description{
 A robust normalization method for zero-inflated count data such as microbiome sequencing data. 
}
\usage{
GMPR(OTUmatrix, min_ct = 2, intersect_no = 4)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{OTUmatrix}{
An OTU count table, where OTUs are arranged in rows and samples in columns.
}
  \item{min_ct}{
The minimal number of OTU counts. Only those OTU pairs with at least \code{min_ct} counts are 
considered in the ratio calculation.   The default is 2.
}
  \item{intersect_no}{
The minimal number of shared OTUs between samples. Only those sample pairs sharing at least \code{intersect_no} OTUs 
are considered in geometric mean calculation. The default is 4.
}
}
\details{
Normalization is a critical step in microbiome sequencing data analysis to account for variable library sizes.
Microbiome data contains a vast number of zeros, which makes the traditional RNA-Seq normalization methods unstable. 
The proposed  GMPR normalization remedies this problem by switching the two steps in DESeq2 normalization:


First, to calculate rij, the median count ratio of nonzero counts between samples:
  rij=median(cki/ckj) (k in 1:OTU_number and cki, ckj is the non-zero count of the kth OTU)

Second, to calculate the size factor si for a given sample i:
  si=geometric_mean(rij)
  
}
\value{
A vector of GMPR size factor for each sample.
}
\references{
Li Chen, James Reeve, Lujun Zhang, Shenbing Huang, and Jun Chen. 2018. GMPR: A robust normalization method for zero-inflated count
data with application to microbiome sequencing data. PeerJ, 6, e4600.
}
\author{
Jun Chen and Lujun Zhang
}


\examples{
data(throat.otu.tab)
size.factor <- GMPR(t(throat.otu.tab))
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ normalization}% use one of  RShowDoc("KEYWORDS")
\keyword{ microbiome }% __ONLY ONE__ keyword per line

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZicoSeq.R
\name{ZicoSeq}
\alias{ZicoSeq}
\title{A linear Model-based Permutation Test for Differential Abundance Analysis of Microbiome Data and Other Omics Data}

\description{ZicoSeq is a permutation test (Smith permutation) for differential abundance analysis of microbiome sequencing data. The input 
can be a count or a proportion matrix.  When a count matrix is provided, it provides an option to draw posterior samples 
of the underlying proportions to account for the sampling variability during the sequencing process. The test results are
aggregated over these posterior samples.  For both count and proportion data, a reference-based ratio approach is used
to account for compositional effects.  As a general methodology, ZicoSeq can also be applied to differential analysis of other
omics data. In this case, they are not treated as compositional data.
}


\usage{
ZicoSeq(
  meta.dat,
  feature.dat,
  grp.name,
  adj.name = NULL,
  feature.dat.type = c('count', 'proportion', 'other'),
  prev.filter = 0,
  mean.abund.filter = 0,
  max.abund.filter = 0,
  min.prop = 0,
  is.winsor = TRUE,
  outlier.pct = 0.03,
  winsor.end = c('top', 'bottom', 'both'),
  is.post.sample = TRUE,
  post.sample.no = 25,
  link.func = list(function(x) sign(x) * (abs(x))^0.5),
  stats.combine.func = max,
  perm.no = 99,
  strata = NULL,
  ref.pct = 0.5,
  stage.no = 6,
  excl.pct = 0.2,
  p.max = 500,
  is.fwer = FALSE,
  verbose = TRUE,
  return.feature.dat = TRUE
)
}
\arguments{
\item{meta.dat}{a data frame containing the sample meta data.}

\item{feature.dat}{a matrix of feature data, row - features (OTUs, genes, etc) , column - samples.}

\item{grp.name}{the name for the variable of interest. It could be numeric or categorical; should be in \code{meta.dat}.}

\item{adj.name}{the name(s) for the variable(s) to be adjusted. Multiple variables are allowed.
       They could be numeric or categorical; should be in \code{meta.dat}.}

\item{feature.dat.type}{the type of the feature data. It could be "count", "proportion" or "other". For "proportion" data type, posterior sampling will not be performed,
but the reference-based ratio approach will still be used to address compositional effects. For "other" data type, neither posterior sampling or reference-base ratio approach
will be used.}

\item{prev.filter}{the prevalence (percentage of nonzeros) cutoff, under which the features will  be filtered. The default is 0. }

\item{mean.abund.filter}{the mean relative abundance cutoff, under which the features will  be filtered. The default is 0.}

\item{max.abund.filter}{the max relative abundance cutoff, under which the features will  be filtered. The default is 0.}

\item{min.prop}{ proportions less than this value will be replaced with this value. Only relevant when log transformation is used.
 Default is 0.}

\item{is.winsor}{a logical value indicating whether winsorization should be performed to replace outliers. The default is TRUE.}

\item{outlier.pct}{the expected percentage of outliers. These outliers will be winsorized. The default is 0.03. For count/proportion data, 
\code{outlier.pct} should be less than \code{prev.filter}.}

\item{winsor.end}{a character indicating whether the outliers at the "top", "bottom" or "both" will be winsorized.
The default is "top". If the \code{feature.dat.type} is "other", "both" may be considered.}

\item{is.post.sample}{a logical value indicating whether to perform posterior sampling of the underlying proportions.
 Only relevant when the feature data are counts.}

\item{post.sample.no}{the number of posterior samples if posterior sampling is used. The default is 25.}

\item{link.func}{a list of transformation functions for the feature data or the ratios. Based on our experience, 
square-root transformation is a robust choice for many datasets.}

\item{perm.no}{the number of permutations. If the raw p values are of the major interest, set \code{perm.no} to at least 999.}

\item{strata}{a factor such as subject IDs indicating the permutation strata or characters indicating the strata variable in \code{meta.dat}. Permutation will be confined to each stratum. This can be used for paired or some longitudinal designs.}

\item{stats.combine.func}{function to combine the F-statistic for the omnibus test. The default is \code{max}.}

\item{ref.pct}{percentage of reference taxa. The default is 0.5.}

\item{p.max}{the maximum number of (most abundant) taxa to be considered in reference taxa selection; only relevant when the nubmer of taxa is huge. The default is 500, i.e., when the nubmer of taxa is larger than 500, only the 500 most abundant taxa will be used for reference selection. This is to reduce the computational time.}

\item{stage.no}{the number of stages if multiple-stage normalization is used. The default is 6.}

\item{excl.pct}{the maximum percentage of significant features (nominal p-value < 0.05) in the reference set that should be removed. 
Only relevant when multiple-stage normalization is used.}

\item{is.fwer}{a logical value indicating whether the family-wise error rate control (West-Young) should be performed.}

\item{verbose}{a logical value indicating whether the trace information should be printed out.}

\item{return.feature.dat}{a logical value indicating whether the winsorized, filtered "feature.dat" matrix should be returned.}

}
\value{
A list with the elements
\item{call}{the call}
\item{feature.dat}{the winsorized, filtered \code{feature.dat} matrix.}
\item{meta.dat}{\code{meta.dat} used.}
\item{grp.name}{the name of the variable of interest.}
\item{filter.features}{a vector of the names of the features that are filtered.}
\item{ref.features}{a vector of the names of the reference features. Only relevant when reference approach is used.}
\item{R2}{a matrix of percent explained variance (number of features by number of transformation functions).}
\item{F0}{a matrix of F-statistics (number of features by number of transformation functions).}
\item{RSS}{a matrix of residual sum squares (number of features by number of transformation functions).}
\item{df.model, df.residual}{degrees of freedom for the model and residual space.}
\item{coef.list}{a list of the linear regression coefficients under the specified transformations.}
\item{p.raw}{the raw p-values based on permutations (not accurate if \code{perm.no} is small).}
\item{p.adj.fdr}{permutation-based FDR-adjusted p-values.}
\item{p.adj.fwer}{permutation-based FWER-adjusted (West-Young) p-values.}
}
\details{

\code{ZicoSeq} is a linear model-based permutation test developed for differential abundance analysis of zero-inflated compositional data. Although its development is
motivated by zero-inflated microbiome sequence count data, it can be applied to proportion (composition) data and more generally to other types of omics data. 
Currently, it has the following components: 1. Winsorization to decrease the influence of outliers; 2. Posterior sampling based on a beta mixture prior to address sampling variability 
 and zero inflation; 3. Reference-based multiple-stage normalization to address compositional effects; 4. An omnibus test to address diverse feature-covariate relationships; 5. Permutation-based 
false discovery rate control / family-wise error rate control for multiple testing correction, which takes into account the correlation structure in the feature data.
}
\references{
Yang, L. & Chen, J. 2022. A comprehensive evaluation of differential abundance analysis methods: current status and potential solutions.
Microbiome, 10(1), 1-23.
}

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{ZicoSeq.plot}}
}

\author{Jun Chen}

\examples{

data(throat.otu.tab)
data(throat.tree)
data(throat.meta)

comm <- t(throat.otu.tab)
meta.dat <- throat.meta

set.seed(123)
# For count data
zico.obj <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm, 
		grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "count",
		# Filter to remove rare taxa
		prev.filter = 0.2, mean.abund.filter = 0,  max.abund.filter = 0.002, min.prop = 0, 
		# Winsorization to replace outliers
		is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
		# Posterior sampling to impute zeros
		is.post.sample = TRUE, post.sample.no = 25, 
		# Multiple link functions to capture diverse taxon-covariate relation
		link.func = list(function (x) x^0.25, function (x) x^0.5, function (x) x^0.75), 
		stats.combine.func = max,
		# Permutation-based multiple testing correction
		perm.no = 99,  strata = NULL, 
		# Reference-based multiple stage normalization
		ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
		# Family-wise error rate control
		is.fwer = FALSE,
		verbose = TRUE, return.feature.dat = FALSE)

which(zico.obj$p.adj.fdr <= 0.05)

# For proportion data
comm.p <- t(t(comm) / colSums(comm))
zico.obj <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm.p, 
		grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "proportion",
		# Filter to remove rare taxa
		prev.filter = 0.2, mean.abund.filter = 0,  max.abund.filter = 0.002, min.prop = 0, 
		# Winsorization to replace outliers
		is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
		# Posterior sampling will be automatically disabled
		is.post.sample = FALSE, post.sample.no = 25, 
		# Use the square-root transformation
		link.func = list(function (x) x^0.5), stats.combine.func = max,
		# Permutation-based multiple testing correction
		perm.no = 99,  strata = NULL, 
		# Reference-based multiple stage normalization
		ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
		# Family-wise error rate control
		is.fwer = FALSE,
		verbose = TRUE, return.feature.dat = FALSE)

which(zico.obj$p.adj.fdr <= 0.05)


# For other type of data. The user should be responsible for the filtering.
comm.o <- comm[rowMeans(comm != 0) >= 0.2, ] + 1
comm.o <- log(t(t(comm.o) / colSums(comm.o)))
zico.obj <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm.o, 
		grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "other",
		# Filter will not be applied
		prev.filter = 0, mean.abund.filter = 0,  max.abund.filter = 0, min.prop = 0, 
		# Winsorization to both ends of the distribution 
		is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'both',
		# Posterior sampling will be automatically disabled
		is.post.sample = FALSE, post.sample.no = 25, 
		# Identity function is used
		link.func = list(function (x) x), stats.combine.func = max,
		# Permutation-based multiple testing correction
		perm.no = 99,  strata = NULL, 
		# Reference-based multiple-stage normalization will not be performed
		ref.pct = 0.5, stage.no = 6, excl.pct = 0.2,
		# Family-wise error rate control
		is.fwer = TRUE,
		verbose = TRUE, return.feature.dat = FALSE)

which(zico.obj$p.adj.fdr <= 0.05)

}

\keyword{composition}
\keyword{microbiome}
\keyword{univariate}
\keyword{permutation}

\name{dICC.SE.bt}
\alias{dICC.SE.bt}


\title{Bootstrap Standard Error of Distance-based Intra-Class Correlation Coefficient}

\description{Calculate the standard error of the distance-based intra-class correlation coefficient based on bootstrapping. Biological replicates are sampled by replacement. 
More conservative than the asymptotic approach.}

\usage{
 dICC.SE.bt(dist.mat, strata, B = 199)
}


		
\arguments{

  \item{dist.mat}{a symmetrical distance matrix between all the replicates (technical and biological replicates).}
  \item{strata}{a factor with each level corresponding to a biological replicate. Supports an unequal number of technical replicates for each biological replicate.}
  \item{B}{integer, the number of bootstrap iterations.}
  

}


\value{

  Function \code{dICC} returns a list with the following component:
  \item{ICC}{the distance-based ICC value.}
   \item{SE}{the standard error.}


}


\references{
Chen, J. & Zhang, X. 2022.  dICC: Distance-based Intraclass Correlation Coefficient for Metagenomic Reproducibility Studies.
submitted.

}

\author{Jun Chen and Xianyang Zhang}


\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{dICC}}, \code{\link{dICC.SE.asympt}}
}

\examples{


# Generate the error-free measurements of 20 biological replicates, each with four dimensions
y <- matrix(rnorm(80), nrow = 20, ncol = 4)

# Generate two technical replicates for each biological replicate and add measurement errors
y1 <- y + matrix(rnorm(80, sd = 0.5), nrow = 20, ncol = 4)
y2 <- y + matrix(rnorm(80, sd = 0.5), nrow = 20, ncol = 4)
y12 <- rbind(y1, y2)

# Create the design vector
strata <- factor(rep(1 : 20, 2))

# Calculate the distance and distance-based ICC
dist.mat <- as.matrix(dist(y12))
dICC.SE.bt(dist.mat, strata)


}





\keyword{distance}
\keyword{multivariate}

\name{dmanova}
\alias{dmanova}


\title{Distance-based Multivariate Analysis of Variance (Analytical P-value Calculation)}

\description{Analysis of variance using distance matrices --- for
  partitioning distance matrices among sources of variation and fitting
  linear models (e.g., factors, polynomial regression) to distance 
  matrices; calculate the analytical p-value based on pseudo-\eqn{F} statistic 
  without permutation.}

\usage{
dmanova(formula, data = NULL, positify = FALSE,
		contr.unordered = "contr.sum", contr.ordered = "contr.poly", 
		returnG = FALSE)
}


		
\arguments{

  \item{formula}{model formula. The LHS must be a dissimilarity matrix (either class \code{matrix} or class
  \code{dist}, e.g., from  \code{\link{vegdist}} or \code{\link{dist}}.  The RHS defines the independent 
  variables. These can be continuous variables or factors, they can be transformed
    within the formula, and they can have interactions as in a typical
    \code{\link{formula}}. }

  \item{data}{the data frame for the independent variables.}
  \item{positify}{a logical value indicating whether to make the Gower's matrix  positive definite
   using the \code{\link{nearPD}} function in \code{Matrix} package. This is equivalent to modifying the distance 
   matrix so that it has an Euclidean embedding. }
    \item{contr.unordered, contr.ordered}{contrasts used for the design
    matrix (default in R is dummy or treatment contrasts for unordered
    factors). }
   \item{returnG}{a logical value indicating whether the Gower's matrix should be returned.}
}

\details{

\code{dmanova} is a permutation-free method for approximating the p-value from  distance-based permutational multivariate
analysis of variance (PERMANOVA). PERMANOVA is  slow when the sample size is large. In contrast,
\code{dmanova} provides an analytical solution, which is several orders of magnitude faster for large sample sizes. The covariate of interest
should be put as the last term in \code{formula} while the variables to be adjusted are put before the covariate of interest.
}

\value{


  Function \code{dmanova} returns a list with the following components:

  \item{aov.tab}{typical AOV table showing sources of variation,
    degrees of freedom,  sums of squares, mean squares,
    \eqn{F} statistics, partial \eqn{R^2}{R-squared} and \eqn{P}
    values. }
  \item{df}{degree of freedom for the Chisquared distribution.} 
  \item{G}{ The Gower's matrix if \code{returnG} is true.}   
  \item{call}{ the call made}

}


\references{
Chen, J. & Zhang, X. 2021.  D-MANOVA: fast distance-based multivariate analysis of variance for large-scale microbiome association studies.
 Bioinformatics. https://doi.org/10.1093/bioinformatics/btab498

}

\author{Jun Chen and Xianyang Zhang}


\examples{
\dontrun{
data(throat.otu.tab)
data(throat.tree)
data(throat.meta)

groups <- throat.meta$SmokingStatus

# Rarefaction
otu.tab.rff <- Rarefy(throat.otu.tab)$otu.tab.rff

# Calculate the UniFrac distance
unifracs <- GUniFrac(otu.tab.rff, throat.tree, alpha=c(0, 0.5, 1))$unifracs

# Test the smoking effect based on unweighted UniFrac distance, adjusting sex
# 'Sex' should be put before 'SmokingStatus'
dmanova(as.dist(unifracs[, , 'd_UW']) ~ Sex + SmokingStatus, data = throat.meta)
}

}

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{adonis3}}
}

\keyword{distance}
\keyword{multivariate}

\name{dICC}
\alias{dICC}


\title{Distance-based Intra-Class Correlation Coefficient}

\description{Distance-based Intra-Class Correlation Coefficient (ICC) is an extension of the traditional univariate ICC to multivariate case, where the relationship between
the multivariate measurements is summarized in a distance matrix. It quantifies the ability of a measurement method in reproducing the inter-sample relationship. }

\usage{
 dICC(dist.mat, strata)
}


		
\arguments{

  \item{dist.mat}{a symmetrical distance matrix between all the replicates (technical and biological replicates).}
  \item{strata}{a factor with each level corresponding to a biological replicate.}

}



\value{

  Function \code{dICC} returns a list with the following component:
  \item{ICC}{the distance-based ICC value.}


}


\references{
Chen, J. & Zhang, X. 2022.  dICC: Distance-based Intraclass Correlation Coefficient for Metagenomic Reproducibility Studies.
submitted.

}

\author{Jun Chen and Xianyang Zhang}


\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{dICC.SE.asympt}}, \code{\link{dICC.SE.bt}}
}

\examples{
# Generate the error-free measurements of 20 biological replicates, each with four dimensions
y <- matrix(rnorm(80), nrow = 20, ncol = 4)

# Generate two technical replicates for each biological replicate and add measurement errors
y1 <- y + matrix(rnorm(80, sd = 0.5), nrow = 20, ncol = 4)
y2 <- y + matrix(rnorm(80, sd = 0.5), nrow = 20, ncol = 4)
y12 <- rbind(y1, y2)

# Create the design vector
strata <- factor(rep(1 : 20, 2))

# Calculate the distance and distance-based ICC
dist.mat <- as.matrix(dist(y12))
dICC(dist.mat, strata)


}


\keyword{distance}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZicoSeq.R
\name{ZicoSeq.plot}
\alias{ZicoSeq.plot}
\title{A Plot Function for Visualizing the ZicoSeq Results}

\description{
\code{ZicoSeq.plot} produces volcano plots with the y-axis being the log10 (adjusted) p-value and the x-axis being the signed  R^2^ 
with the sign indicating the association direction determined based on  the sign of the regression coefficients (for multi-categorical
 variables, sign is not applicable). The names of differential taxa passing a specific cutoff will be printed on the figure.
  When data types are counts and proportions, the mean abundance and prevalence will be visualized; when the data type is `other`, 
  mean and standard deviation of the features will be visualized. Users need to set \code{return.feature.dat = T} when using the plot function.
}


\usage{
ZicoSeq.plot(
  ZicoSeq.obj,
  pvalue.type = c('p.adj.fdr','p.raw','p.adj.fwer'), 
  cutoff = 0.1, 
  text.size = 10,
  out.dir = NULL, 
  file.name = 'ZicoSeq.plot.pdf', 
  width = 10, 
  height = 6)
}
\arguments{

\item{ZicoSeq.obj}{object from calling the function \code{ZicoSeq}.}
\item{pvalue.type}{character string, one of 'p.adj.fdr','p.raw' and 'p.adj.fwer'.}
\item{cutoff}{a cutoff between 0 and 1 for pvalue.type, below which the names of the features will be printed.}
\item{text.size}{text size for the plots.}
\item{out.dir}{character string; the directory to save the figure, e.g., \code{getwd()}. Default is NULL. If NULL, figure will not be saved.}
\item{file.name }{character string; name of the file to be saved. }
\item{width}{the width of the graphics region in inches. See R function \code{ggsave}.}
\item{height}{the height of the graphics region in inches. See R function \code{ggsave}.}

}
\value{
gtable of aligned plots from \code{ggarrange}.
}

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{ZicoSeq}}
}

\references{
Yang, L. & Chen, J. 2022. A comprehensive evaluation of differential abundance analysis methods: current status and potential solutions.
Microbiome. Microbiome, 10(1), 1-23.
}


\author{Lu Yang, Jun Chen}

\examples{

data(throat.otu.tab)
data(throat.tree)
data(throat.meta)

comm <- t(throat.otu.tab)
meta.dat <- throat.meta

set.seed(123)
# For count data
zico.obj <- ZicoSeq(meta.dat = meta.dat, feature.dat = comm, 
		grp.name = 'SmokingStatus', adj.name = 'Sex', feature.dat.type = "count",
		# Filter to remove rare taxa
		prev.filter = 0.2, mean.abund.filter = 0,  max.abund.filter = 0.002, min.prop = 0, 
		# Winsorization to replace outliers
		is.winsor = TRUE, outlier.pct = 0.03, winsor.end = 'top',
		# Posterior sampling to impute zeros
		is.post.sample = TRUE, post.sample.no = 25, 
		# Multiple link functions to capture diverse taxon-covariate relation
		link.func = list(function (x) x^0.25, function (x) x^0.5, function (x) x^0.75), 
		stats.combine.func = max,
		# Permutation-based multiple testing correction
		perm.no = 99,  strata = NULL, 
		# Reference-based multiple stage normalization
		ref.pct = 0.5, stage.no = 6, excl.pct = 0.2, 
		# Family-wise error rate control
		is.fwer = FALSE,
		verbose = TRUE, return.feature.dat = TRUE)

which(zico.obj$p.adj.fdr <= 0.1)

ZicoSeq.plot(ZicoSeq.obj = zico.obj, pvalue.type = 'p.adj.fdr', 
             cutoff = 0.1, text.size = 10, out.dir = NULL, width = 15, height = 10)    

}

\keyword{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.R
\name{SimulateMSeqC}
\alias{SimulateMSeqC}
\title{A Semiparametric Model-based Microbiome Sequencing Data Simulator for Longitudinal, Matched-pair, and Replicate Sampling Designs}
\description{
The function generates synthetic microbiome sequencing data for studying the performance of differential abundance analysis methods for correlated microbiome data 
generated in longitudinal, matched-pair and replicate sampling study designs. 
It uses a user-supplied (large) reference OTU table as a template to generate a synthetic OTU table of specified size. A subset of OTUs can be affected by
by a binary variable (group effect) and/or a time variable (temporal effect). Time X group interaction and confounder effects can also be simulated. 
 The function allows simulating different signal structures, i.e., the percentage of differential OTUs, their effect sizes, and their direction of change.
}

\usage{
SimulateMSeqC(	
  ref.otu.tab,
  model.paras = NULL,
  nSubject = 40,
  nOTU = 50,
  nTime = 2, 
  error.sd = 1, 
  MgX = 0.5, 
  SgX = 0, 
  X.diff.otu.pct = 0.1,
  grp.ratio = 1, 
  balanced.X = TRUE,
  MgT = 0, 
  SgT = 0, 
  SbT = 0, 
  T.diff.otu.pct = 0, 
  balanced.T = TRUE,
  MgXT = 0, 
  SgXT = 0, 
  XT.diff.otu.pct = 0, 
  balanced.XT = TRUE,
  conf.cov.cor = 0.6,
  confounder = c('X', 'T'),
  MgZ = 0.5,
  SgZ = 0, 
  Z.diff.otu.pct = 0.05,
  Z.nondiff.otu.pct = 0.1, 
  depth.mu = 10000,
  depth.theta = 5,
  depth.conf.factor = 0
)
}
\arguments{
\item{ref.otu.tab}{a matrix, the reference OTU count table (row - OTUs, column - samples), serving as the template for synthetic sample generation.}

\item{model.paras}{a list of Dirichlet-Multinomial parameters from running \code{GUniFrac:::EstPara(ref.otu.tab)}. As MLE-based parameter estimation is computationally intensive, 
for simulation studies with multiple iterations,  the parameters can be estiamted once and used repeatedly.}


\item{nSubject}{the number of subjects to be simulated.}

\item{nOTU}{the number of OTUs to be simulated. }

\item{nTime}{the number of time points to be simulated. }

\item{error.sd}{the standar deviation of the random error controlling the within-subject correlation strength. Large k = 1, small k = 4.}

\item{MgX}{a numeric value indicating the mean group (X) effect (log fold change ) across the associated OTUs. The default is 0.}

\item{SgX}{a numeric value indicating the variance of group (X) effect (log fold change) across the associated OTUs. The default is 0.}

\item{X.diff.otu.pct}{a numeric value between 0 and 1, the percentage of differential OTUs regarding the group (X) to be simulated.  The default is 0.1.}

\item{grp.ratio}{a numeric value between 0 and 1. Group size ratio. The default is 1, i.e., equal group size. }

\item{balanced.X}{a logical value.  TRUE - the direction of change for these group differential OTUs is random, FALSE - the direction of change is the same. The default is "balanced".}

\item{MgT}{a numeric value indicating the population mean of the time (T) effect (log fold change) across the associated OTUs. The default is 0.}

\item{SgT}{a numeric value indicating the population variance of the time (T) effect (log fold change) across the associated OTUs. The default is 0.}

\item{SbT}{a numeric value indicating the variance of time (T) effect (log fold change) across the subjects. This parameter is to generate a subject-level random slope 
(temporal trends differ by subjects). The default is 0. }

\item{T.diff.otu.pct}{a numeric value between 0 and 1, the percentage of time differential OTUs  to be simulated.  The default is 0.}

\item{balanced.T}{a logical value.  TRUE - the direction of change for these time differential OTUs is random, FALSE - the direction of change is the same. The default is "balanced".}

\item{MgXT}{a numeric value indicating the mean X:T interaction effect (log fold change) across the associated OTUs. The default is 0.}

\item{SgXT}{a numeric value indicating the variance of X:T interaction effect (log fold change) across the associated OTUs. The default is 0.}

\item{XT.diff.otu.pct}{a numeric value between 0 and 1, the percentage of X:T interaction OTUs to be simulated.  The default is 0.}

\item{balanced.XT}{a logical value.  TRUE - the direction of change for the interaction effects is random, FALSE - the direction of change is the same. The default is "balanced".}

\item{conf.cov.cor}{a numeric value between 0 and 1.  The correlation strength between the group and the confounder. The default is 0.6.}

\item{confounder}{one of 'X' or 'T', whether the confounder is correlated with 'X' or 'T'.}

\item{MgZ}{a numeric value indicating the mean confounder (Z) effect (log fold change) across the associated OTUs. The default is 0.}

\item{SgZ}{a numeric value indicating the variance of confounder (Z) effect (log fold change) across the associated OTUs. The default is 0.}

\item{Z.diff.otu.pct}{a numeric value between 0 and 1, less than the percentage of group differential OTUs,
 the percentage of confounder-associated OTUs, which are also group differential.  The default is 0.05.}

\item{Z.nondiff.otu.pct}{a numeric value between 0 and 1, less than the percentage of group non-differential OTUs,
the percentage of confounder-associated OTUs, which are not group differential.  The default is 0.1.}


\item{depth.mu}{the mean sequencing depth to be simulated. The default is 10,000.}

\item{depth.theta}{the theta value  of the negative binomial distribution controlling the variance (\code{mu + mu^2/theta}). The default is 5.}

\item{depth.conf.factor}{a numeric value controlling the dependence of the sequencing depth on the group variable (\code{depth.mu} * \code{exp(scale(X)} * \code{depth.conf.factor})). 
The default is 0, i.e., the depth is not different between groups. This parameter can be used to simulate depth confounding.}
}

\value{
Return a list with the elements:
\item{otu.tab.sim}{simulated OTU table}
\item{meta}{meta data containing the simulated covariates (group, time, confounder)}
\item{otu.names}{the names of the simulated OTUs}
\item{X.diff.otu.ind}{indices of the group differential OTUs}
\item{T.diff.otu.ind}{indices of the time differential OTUs}
\item{XT.diff.otu.ind}{indices of the OTUs with group:time interaction}
\item{Z.diff.otu.ind}{indices of OTUs affected by the confounder}
}

\details{

This function implements a semiparametric approach for realistic correlated microbiome data generation. The method draws random samples from a large reference dataset (non-parametric part) and uses
 these reference samples as templates to generate new samples (parametric part). Specifically, for each drawn reference sample, it infers the underlying composition based on a Bayesian model and 
 then adds group/time/group:time/confounder effects to the composition vector, based on which a new sequencing sample is generated. The method circumvents the difficulty in modeling the inter-subject variation
 of the microbiome composition.   

}
\references{
Yang, L. & Chen, J. Benchmarking Differential Abundance Analysis Methods for Correlated Microbiome Sequencing Data. Briefings in bioinformatics 24.1 (2023): bbac607.

}


\author{Lu Yang and Jun Chen}

\examples{

# Use throat microbiome for illustration
data(throat.otu.tab)
comm <- t(throat.otu.tab)
comm <- comm[rowMeans(comm != 0) > 0.2, ]

# Example1: Simulate replicate sampling data, 40 subjects each with two replicates (nTime =2), 
# two group comparison, 10\% group differential OTUs
\dontrun{
sim.obj <- SimulateMSeqC(ref.otu.tab= comm, 
                         nSubject = 40, nOTU = 50, nTime = 2,
                         # Within-subject correlation setting
                         error.sd = 1, 
                         # Group effect setting, unbalanced
                         MgX = 0.5, SgX = 0, X.diff.otu.pct = 0.1, grp.ratio = 1, 
                         balanced.X = FALSE,
                         # Time effect setting (No time effect)
                         MgT = 0, SgT = 0, SbT = 0, T.diff.otu.pct = 0,
                         # Interaction effect setting (No interaction effect)
                         MgXT = 0, SgXT = 0, XT.diff.otu.pct = 0, 
                         # Confounder effect setting
                         conf.cov.cor = 0.6, confounder = 'X',
                         MgZ = 0.5, SgZ = 0,  Z.diff.otu.pct = 0.05, Z.nondiff.otu.pct = 0.1, 
                         # Sequencing Depth setting
                         depth.mu = 10000, depth.theta = 5,  depth.conf.factor = 0)
}
# Example2: Simulate matched-pair data, 100 subjects each with pre- and post-treatment (nTime = 2), 
# 10\% differential OTUs
\dontrun{
sim.obj <- SimulateMSeqC(ref.otu.tab= comm, 
                         nSubject = 100, nOTU = 50, nTime = 2,
                         # Within-subject correlation setting
                         error.sd = 1, 
                         # Group effect setting (No group effect)
                         MgX = 0, SgX = 0, X.diff.otu.pct = 0, grp.ratio = 1, 
                         # Time effect setting (No random slope, SbT=0)
                         MgT = 0.5, SgT = 0, SbT = 0, T.diff.otu.pct = 0.1, 
                         # Interaction effect setting (No interaction effect)
                         MgXT = 0, SgXT = 0, XT.diff.otu.pct = 0,
                         # Confounder effect setting (T!)
                         conf.cov.cor = 0.6, confounder = 'T',
                         MgZ = 0, SgZ = 0,  Z.diff.otu.pct = 0.05, Z.nondiff.otu.pct = 0.1, 
                         # Sequencing Depth setting
                         depth.mu = 10000, depth.theta = 5,  depth.conf.factor = 0)
}

# Example3: Simulate the general longitudinal data, 40 Subjects each with five time points, 
# two groups, 10\% group differential OTUs, 10 \% time differential OTUs and 10 \% interaction OTUs.
\dontrun{
sim.obj <- SimulateMSeqC(ref.otu.tab= comm, 
                         nSubject = 40, nOTU = 50, nTime = 5,
                         # Within-subject correlation setting
                         error.sd = 1, 
                         # Group effect setting, balanced
                         MgX = 0.5, SgX = 0, X.diff.otu.pct = 0.1, grp.ratio = 1, 
                         balanced.X = TRUE,
                         # Time effect setting (random slope)
                         MgT = 0.5, SgT = 0, SbT = 0.5, T.diff.otu.pct = 0.1, 
                         # Interaction effect setting 
                         MgXT = 0.5, SgXT = 0, XT.diff.otu.pct = 0.1, 
                         # Confounder effect setting
                         conf.cov.cor = 0.6, confounder = 'X',
                         MgZ = 0.5, SgZ = 0,  Z.diff.otu.pct = 0.05, Z.nondiff.otu.pct = 0.1, 
                         # Depth setting
                         depth.mu = 10000, depth.theta = 5,  depth.conf.factor = 0)
}
}
\keyword{composition}
\keyword{microbiome}
\keyword{simulation}

\name{throat.tree}
\alias{throat.tree}
\docType{data}
\title{
%%   ~~ data name/kind ... ~~
UPGMA Tree of Throat Microbiome OTUs
}
\description{
%%  ~~ A concise (1-5 lines) description of the dataset. ~~
The OTU tree is constructed using UPGMA on the K80 distance matrice of
the OTUs. It is a rooted tree of class "phylo".
}
\usage{data(throat.tree)}

\details{
%%  ~~ If necessary, more details than the __description__ above ~~
The OTUs are produced by the QIIME software. Singleton OTUs have
been discarded.
}
\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
Charlson ES, Chen J, Custers-Allen R, Bittinger K, Li H, et al. (2010) 
Disordered Microbial Communities in the Upper Respiratory Tract of 
Cigarette Smokers. PLoS ONE 5(12): e15216. 
}

\examples{
data(throat.tree)
}
\keyword{datasets}

\name{Rarefy}
\alias{Rarefy}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Rarefy a Count Table to Equal Sequencing Depth
}
\description{
GUniFrac is also sensitive to different sequencing depth. To compare
microbiomes on an equal basis, rarefaction might be used.
}
\usage{
Rarefy(otu.tab, depth = min(rowSums(otu.tab)))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{otu.tab}{
OTU count table, row - n sample, column - q OTU
}
  \item{depth}{
required sequencing depth; If not specified, the lowest sequencing
depth is used.
}
}

\value{
 Return a list containing:
 \item{otu.tab.rff}{rarefied OTU table}
 \item{discard}{IDs of samples that does not reach the specified sequencing depth}
%% ...
}
\references{
%% ~put references to the literature/web site here ~
Chen, J., Bittinger, K., Charlson, E.S., Hoffmann, C., Lewis, J., Wu, G.D., Collman, R.G., Bushman, F.D. and Li, H. (2012). Associating microbiome composition with 
environmental covariates using generalized UniFrac distances. 28(16): 2106–2113.
}
\author{
%%  ~~who you are~~
Jun Chen <chen.jun2@mayo.edu>
}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\examples{
data(throat.otu.tab)
# Rarefaction
otu.tab.rff <- Rarefy(throat.otu.tab, 1024)$otu.tab.rff
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ Normalization}
\keyword{ Microbiome}% __ONLY ONE__ keyword per line

\name{vaginal.otu.tab}
\alias{vaginal.otu.tab}
\docType{data}
\title{
%%   ~~ data name/kind ... ~~
Vaginal Microbiome OTU Count Table
}
\description{
%%  ~~ A concise (1-5 lines) description of the dataset. ~~
OTU count table from 16S V3-V5 targeted sequencing of the vaginal microbiome samples from the HMP project. A total of 780 OTUs from 381 samples.
}
\usage{data(vaginal.otu.tab)}

\details{
%%  ~~ If necessary, more details than the __description__ above ~~
The OTU table was taken from R bioconductor "HMP16SData" package. OTUs with prevalence less than 10\% and maximum proportion less than 0.2\% were removed. 
This OTU table can be used for simulating vaginal microbiome sequencing data.
}
\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
Schiffer L, Azhar R, Shepherd L, Ramos M, Geistlinger L, Huttenhower C, Dowd JB, Segata N, Waldron L (2019). 
“HMP16SData: Efficient Access to the Human Microbiome Project through Bioconductor.” American Journal of Epidemiology. doi: 10.1093/aje/kwz006.
}

\examples{
data(stool.otu.tab)
}
\keyword{datasets}

\name{GUniFrac}
\alias{GUniFrac}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Generalized UniFrac distances for comparing microbial communities.
}
\description{
A generalized version of commonly used UniFrac distances. It is defined as:
\deqn{d^{(\alpha)} = \frac{\sum_{i=1}^m b_i (p^A_{i} + p^B_{i})^\alpha 
\left\vert \frac{ p^A_{i} - p^B_{i} }{p^A_{i} + p^B_{i}} \right\vert } { 
 \sum_{i=1}^m b_i (p^A_{i} + p^B_{i})^\alpha},}{d^{(\alpha)} = 
 [\sum_{i=1}^m b_i (p^A_{i} + p^B_{i})^\alpha 
|p^A_{i} - p^B_{i}|/(p^A_{i} + p^B_{i})]/ [ 
 \sum_{i=1}^m b_i (p^A_{i} + p^B_{i})^\alpha],}
 
where \eqn{m}{m} is the number of branches, \eqn{b_i}{b_i} is the length of 
\eqn{i}{i}th branch, \eqn{p^A_{i}, p^B_{i}}{p^A_{i}, p^B_{i}} are the branch
proportion for community A and B. 

Generalized UniFrac distance contains an extra parameter \eqn{\alpha}{\alpha} 
controlling the weight on abundant lineages so the distance is not dominated 
by highly abundant lineages. \eqn{\alpha=0.5}{\alpha=0.5} ("d_0.5") is overall very robust. 

The unweighted ("d_1") and weighted UniFrac ("d_UW") are also implemented.  
}

\usage{
GUniFrac(otu.tab, tree, size.factor = NULL, alpha = c(0, 0.5, 1), verbose = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{otu.tab}{
%%     ~~Describe \code{otu.tab} here~~
a matrix, the OTU count table, row - n sample, column - q OTU
}
  \item{tree}{
%%     ~~Describe \code{tree} here~~
a rooted phylogenetic tree of R class \dQuote{phylo}
}

  \item{size.factor}{
   a numeric vector of the normalizing factors to divide the counts. The length
   is the number of samples. This provides the flexibility to normalize data using the 
   preferred normalization method (e.g. GMPR normalizing factor). If not supplied, the total sum will be used. 
}
  \item{alpha}{
%%     ~~Describe \code{alpha} here~~
a numeric vector, parameters controlling the weight on abundant lineages
}

  \item{verbose}{
%%     ~~Describe \code{alpha} here~~
logical value, whether to print out the messages
}
}

\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
Return a list containing
		\item{unifracs}{a three dimensional array containing all the UniFrac
		distance matrices}
}
\references{
%% ~put references to the literature/web site here ~
Chen, J., Bittinger, K., Charlson, E.S., Hoffmann, C., Lewis, J., Wu, G.D., Collman, R.G., Bushman, F.D. and Li, H. (2012). Associating microbiome composition with 
environmental covariates using generalized UniFrac distances. 28(16): 2106–2113.
}
\author{
%%  ~~who you are~~
Jun Chen <chen.jun2@mayo.edu>
}

\note{
%%  ~~further notes~~
The function only accepts rooted tree. To root a tree, you may consider using \code{midpoint} from the package \code{phangorn}.
}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{Rarefy}}, \code{\link{PermanovaG}}
}
\examples{
\dontrun{
data(throat.otu.tab)
data(throat.tree)
data(throat.meta)

groups <- throat.meta$SmokingStatus

# Rarefaction
otu.tab.rff <- Rarefy(throat.otu.tab)$otu.tab.rff

# Calculate the UniFracs
unifracs <- GUniFrac(otu.tab.rff, throat.tree, alpha=c(0, 0.5, 1))$unifracs

dw <- unifracs[, , "d_1"]		# Weighted UniFrac
du <- unifracs[, , "d_UW"]		# Unweighted UniFrac	
d0 <- unifracs[, , "d_0"]     	# GUniFrac with alpha 0  
d5 <- unifracs[, , "d_0.5"]   	# GUniFrac with alpha 0.5 

# Permanova - Distance based multivariate analysis of variance
adonis3(as.dist(d5) ~ groups)
}

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ distance }
\keyword{ UniFrac }
\keyword{ ecology }% __ONLY ONE__ keyword per line
